/* GANG Software
 * GEOM/geom/io/MathWrite.C
 * Copyright (C) 2002 Nicholas Schmitt <nick@gang.umass.edu>
 * Wed Sep 18 16:39:46 2002
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "MathWrite.H"

namespace geom {
namespace io {

MathWrite::
MathWrite()
    :
    Writer(),
    _stream((::io::Ostream *)0),
    _geomlist((::geom::surface::SurfaceBase const *)0),
    _milieu((milieu::kernel::MilieuBase const *)0)
{
}

MathWrite::
~MathWrite()
{
}

void
MathWrite::
write(
  ::io::Ostream &o,
  ::geom::surface::SurfaceBase const &g,
  milieu::kernel::MilieuBase const &M ) const
{
  _stream = &o;
  _geomlist = &g;
  _milieu = &M;

  _stream->mode( ::io::MATHEMATICA );

  write_header();

  std::list<surface0::GeomObject *>::const_iterator i;
  for ( i = g.list().begin(); i != g.list().end(); i++ )
  {
    write_part( *i );
  }

  write_footer();

  _stream = (::io::Ostream *)0;
  _geomlist = (::geom::surface::SurfaceBase const *)0;
  _milieu = (milieu::kernel::MilieuBase const *)0;
}

void
MathWrite::
write_header() const
{
  if ( ! _geomlist->name().empty() ) {
    *_stream << "(* " << _geomlist->name() << " *)\n";
  }
  *_stream << "(* " << "Generated by GANGLab Software v"
    << base::system->version() << " *)\n"
    << "(*   " << datestamp_utc() << " *)\n\n";
}
void
MathWrite::
write_footer() const
{
}

void
MathWrite::
write_part(
  surface0::GeomObject const *g ) const
{
  if ( g->dimension() == 2U )
  {
    write_part( static_cast<surface0::Surface const *>(g) );
  }
}

void
MathWrite::
write_part(
  surface0::Surface const *g ) const
{
  _write_normal0 = _write_normal == 1 ? true : false;
  _write_color0 = _write_color == 1 ? true : false;
  _write_color0 &= g->has_array_color();

  g->compute_normal();

  *_stream << "Graphics3D[{\n";

  geom::Vertex3 v;

  uint i;
  for ( i = 0; i < g->face_count(); i++ )
  {
    // write a face
    *_stream << "Polygon[{\n";
    uint n = g->face_count(i);
    uint j;
    for ( j = 0; j < n; j++ )
    {
      // write a vertex
      g->vertex( v, g->face_vertex(i,j) );
      *_stream << "{" << v[0] << ", " << v[1] << ", " << v[2] << "}";
      if (j != n-1) { *_stream << ","; }
      *_stream << "\n";
    }
    *_stream << "}]";
    if (i != g->face_count()-1) { *_stream << ","; }
    *_stream << "\n";
  }
  *_stream << "},\n";

  *_stream <<
    "{PlotRange -> Automatic, DisplayFunction :> $DisplayFunction,\n"
    "ColorOutput -> Automatic, Axes -> True, PlotLabel -> None,\n"
    "AxesLabel -> None, Ticks -> Automatic, Prolog -> {}, Epilog -> {},\n"
    "AxesStyle -> Automatic, Background -> Automatic, DefaultColor -> Automatic,\n"
    "DefaultFont :> $DefaultFont, AspectRatio -> Automatic,\n"
    "ViewPoint -> {1.3, -2.4, 2.}, Boxed -> True, BoxRatios -> Automatic,\n"
    "Plot3Matrix -> Automatic, Lighting -> True, AmbientLight -> GrayLevel[0.],\n"
    "LightSources -> {{{1., 0., 1.}, RGBColor[1, 0, 0]},\n"
    "  {{1., 1., 1.}, RGBColor[0, 1, 0]}, {{0., 1., 1.}, RGBColor[0, 0, 1]}},\n"
    "ViewCenter -> Automatic, PlotRegion -> Automatic, ImageSize -> Automatic,\n"
    "TextStyle :> $TextStyle, FormatType :> $FormatType,\n"
    "ViewVertical -> {0., 0., 1.}, FaceGrids -> None, Shading -> True,\n"
    "RenderAll -> True, PolygonIntersections -> True, AxesEdge -> Automatic,\n"
    "BoxStyle -> Automatic, SphericalRegion -> False}]\n";

}

} // namespace io
} // namespace geom
