/* GANG Software
 * VIEWER/isight/context/Stars.C
 * Copyright (C) 2002 Nicholas Schmitt <nick@gang.umass.edu>
 * Wed Sep 18 16:39:46 2002
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>
#include <GL/gl.h>
#include "base/System.H"
#include "io/BinaryIstream.H"
#include "Stars.H"

namespace isight {
namespace context {

base::vector<Stars::StarData> Stars::_stardata;

void
Stars::
initialize()
{
  typedef char unsigned uchar;

  uint STAR_COUNT = 1119;
  char const *d = _data;
  char const *d1 = _data + 16 * STAR_COUNT;
  for (;;)
  {
    StarData s;
    s.position[0] = *(float *)(d);
    d += 4;
    s.position[1] = *(float *)(d);
    d += 4;
    s.position[2] = *(float *)(d);
    d += 4; 
    s.size = float(*(uchar *)(d))/8.0;
    d += 1;
    s.color[0] = float(*(uchar *)(d))/255.0f;
    d += 1;
    s.color[1] = float(*(uchar *)(d))/255.0f;
    d += 1;
    s.color[2] = float(*(uchar *)(d))/255.0f;
    d += 1;
    _stardata.push_back(s);
    if ( d == d1) { break; }
  }
}

Stars::
Stars()
    :
    _enable(false)
{
  static int first = true;
  if (first)
  {
    initialize();
    first = false;
  }
}

Stars::
~Stars()
{
}

void
Stars::
enable(bool state)
{
  _enable = state;
  glClearColor( 0.0f, 0.0f, 0.0f, 1.0 );
  glClear(GL_COLOR_BUFFER_BIT);
}

void
Stars::
draw() const
{
  if ( ! _enable ) { return; }

  // black background
  glClearColor( 0.0f, 0.0f, 0.0f, 1.0 );
  glClear(GL_COLOR_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

//  glPushAttrib(GL_ALL_ATTRIB_BITS);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  int i;
  int n = _stardata.size();

  for( i = 0; i < n; i++ )
  {
    StarData const &s = _stardata[i];

    float t = s.size;
    // set point size
    glPointSize( t );

    // set color, modified
    float a = (t < 1 ? t : 1 - (ceil(t) - t) * 0.25f);
    glColor3f( a * s.color[0], a * s.color[1], a * s.color[2] );

    // draw points
    glBegin(GL_POINTS);
    glVertex3fv( s.position );
    glEnd();
  }

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
//  glPopAttrib();
  glPopMatrix();
}

char const *Stars::_data =
"\x2b\x56\xae\x3d\x59\xfc\x23\x3e\x6c\xc1\x7b\xbf\x01\xff\xff\xff"
"\x4b\xaf\xc6\xbe\x3b\x6b\xd4\xbe\xcd\xae\x52\xbf\x01\xff\xff\xff"
"\x73\x01\x24\xbf\xe9\x64\x9a\xbd\xdf\x9d\x43\xbf\x01\xff\xff\xff"
"\x09\x60\x64\xbf\x05\xec\x9b\x3d\xa4\x0a\xe4\xbe\x01\xff\xff\xff"
"\xb3\x96\xc6\xbd\xb5\xc0\x73\xbf\xac\x63\x94\x3e\x01\xff\xff\xff"
"\x93\xc9\x82\x3e\xc1\x25\x89\xbd\xd4\xe9\x76\x3f\x01\xff\xff\xff"
"\xcf\x49\x6b\x3e\x81\x24\x74\x3f\x57\xd6\x46\x3e\x01\xff\xff\xff"
"\x8a\xb3\x3b\x3f\x1f\x40\x92\x3e\x64\xfa\x1d\x3f\x01\xff\xff\xff"
"\x76\x5f\x54\x3f\x81\xfc\xa6\xbe\x56\x0e\xe8\x3e\x01\xff\xff\xff"
"\xb7\x06\x83\x3e\xdf\x02\x75\xbf\xe2\x60\x0b\xbe\x01\xff\xff\xff"
"\x06\x93\x23\x3f\x2b\x55\x13\xbf\xc4\xa9\x02\xbf\x01\xff\xff\xff"
"\x74\x87\x78\x3e\xf4\x4a\x13\x3e\xf7\x99\x75\xbf\x01\xff\xff\xff"
"\x2a\x0c\x43\x3e\x4f\x75\x29\x3e\xaf\xb7\x77\xbf\x01\xff\xff\xff"
"\x1d\xff\x05\xbf\xf7\x46\x4e\xbf\x59\xdc\x8d\xbe\x01\xff\xff\xff"
"\x7e\x43\x48\xbf\x1f\xb8\xfa\xbe\x98\x22\xc5\xbe\x01\xff\xff\xff"
"\x1b\xf3\x53\xbf\x62\x33\xc6\xbe\x1d\xc6\xcf\xbe\x01\xff\xff\xff"
"\x00\x52\xfc\xbe\x72\x77\x54\xbf\x5b\xd5\x85\x3e\x01\xff\xff\xff"
"\x9e\xb8\x57\xbf\xd5\x64\x65\xbc\x7d\xcb\x09\x3f\x01\xff\xff\xff"
"\x3f\x1f\x0f\x3e\xbe\x2a\xd1\xbe\x64\xe8\x66\x3f\x01\xff\xff\xff"
"\xd9\x99\x02\x3f\xc4\x21\x0d\x3f\x88\x00\x29\x3f\x01\xff\xff\xff"
"\xaf\x69\x55\x3f\x8d\x89\x45\x3e\x07\x7c\x04\x3f\x01\xff\xff\xff"
"\x46\xfe\x61\x3f\xab\xdc\x05\xbe\xec\x05\xe7\x3e\x01\xff\xff\xff"
"\x11\x2e\x70\x3f\x54\xa2\x38\xbd\xff\xae\xaf\x3e\x01\xff\xff\xff"
"\x9b\x38\xf4\x3e\x7a\xd9\x55\xbf\xcf\xe8\x8b\x3e\x01\xff\xff\xff"
"\x59\xda\x6a\x3f\x79\x82\xb1\xbe\xf4\x18\x48\x3e\x01\xff\xff\xff"
"\xd5\x86\x67\x3f\x5e\x3b\xd5\xbe\xd6\x06\xbe\x3d\x01\xff\xff\xff"
"\x7b\xb9\xbf\x3d\x78\xb6\x87\x3e\x84\xad\x75\xbf\x02\xff\xff\xff"
"\x93\xca\xe3\xbe\x31\xbc\x06\xbf\x63\x80\x39\xbf\x02\xff\xff\xff"
"\xc4\x1a\x9b\xbe\xa2\x95\x35\x3f\x47\xf0\x22\xbf\x02\xff\xff\xff"
"\x90\x51\x01\xbf\x02\xd7\x06\xbf\xb5\x04\x2f\xbf\x02\xff\xff\xff"
"\x09\x43\x40\xbf\x74\x30\x14\xbf\xa9\x9e\xa2\x3e\x02\xff\xff\xff"
"\x78\x79\xab\xbe\x80\x10\x1e\xbf\x22\x36\x36\x3f\x02\xff\xff\xff"
"\x25\x65\xbc\xbe\xbc\x74\xa8\xbe\xf3\xa3\x5e\x3f\x02\xff\xff\xff"
"\xdc\x6f\x2d\x3e\xea\xf5\x2e\x3e\x6e\x7b\x78\x3f\x02\xff\xff\xff"
"\x57\x01\x4e\x3e\x16\xdc\x78\x3e\x53\xed\x72\x3f\x02\xff\xff\xff"
"\x93\x15\xc8\x3d\xd0\xea\x56\xbf\x83\xd3\x08\x3f\x02\xff\xff\xff"
"\x64\xc4\x48\x3e\x17\x12\x1b\xbf\x9e\x68\x45\x3f\x02\xff\xff\xff"
"\xcc\x42\x16\x3f\x03\x01\x02\x3f\x05\x6c\x21\x3f\x02\xff\xff\xff"
"\x4c\x4a\x52\x3f\x09\x1a\xaa\x3e\x0b\x52\xed\xbe\x02\xff\xff\xff"
"\x8c\xd3\xa4\x3e\x32\x6d\x04\xbf\x56\xff\x4a\xbf\x02\xff\xff\xff"
"\x06\x30\x5a\x3e\x68\x47\x16\x3e\xd0\x48\x77\xbf\x02\xff\xff\xff"
"\x72\xa2\xfd\x3d\x49\xb3\x8a\x3d\x17\x70\x7d\xbf\x02\xff\xff\xff"
"\xb2\xad\xd4\xbd\xef\x07\x6e\xbf\x40\xca\xb4\xbe\x02\xff\xff\xff"
"\x2d\x8c\xbe\xbe\xe5\x2b\x1a\x3e\xf9\x76\x6a\xbf\x02\xff\xff\xff"
"\xda\x9f\xf5\xbe\x34\xea\x10\xbf\xb6\x9e\x2b\xbf\x02\xff\xff\xff"
"\x53\x7f\x0d\xbf\x56\xbd\x48\xbf\xdc\x79\x90\x3e\x02\xff\xff\xff"
"\x06\xdc\x09\xbf\x54\x29\x37\xbf\x6a\xe2\xe3\x3e\x02\xff\xff\xff"
"\x72\x73\x98\xbe\xd8\xd9\x70\x3f\x3b\xc0\x25\x3e\x02\xff\xff\xff"
"\x0e\x3d\x82\xbe\xea\xa4\x2d\xbf\xa2\x79\x30\x3f\x02\xff\xff\xff"
"\x50\x40\x15\xbe\x60\xfb\x9f\xbd\xf5\x79\x7c\x3f\x02\xff\xff\xff"
"\xa9\xcf\x4e\x3c\x65\x86\x2f\x3f\x73\x53\x3a\x3f\x02\xff\xff\xff"
"\x51\x70\x47\x3e\xdf\x24\xbe\xbe\x75\x67\x68\x3f\x02\xff\xff\xff"
"\x56\xb1\x2c\x3f\x48\xda\x3c\x3f\x4c\xe2\xdd\xbc\x02\xff\xff\xff"
"\x2a\x3f\x58\x3f\xe8\x3d\x8c\x3e\xdb\x6c\xeb\xbe\x02\xff\xff\xff"
"\x17\xdf\x15\x3f\x7c\xdd\xba\x3e\xf8\x52\x39\xbf\x02\xff\xff\xff"
"\x56\x2e\x9a\x3e\x4f\x1f\x02\xbf\x31\x8c\x4e\xbf\x02\xff\xff\xff"
"\xe3\x1c\xa6\xbe\xce\xe3\xb6\x3d\x45\x12\x71\xbf\x02\xff\xff\xff"
"\x79\xe6\xe1\xbe\x05\x19\x01\xbf\x81\x08\x3e\xbf\x02\xff\xff\xff"
"\xcc\x16\xd5\xbe\x0c\x52\x26\xbf\x62\xdb\x22\xbf\x02\xff\xff\xff"
"\x91\xe0\x32\xbf\x9a\x1a\xab\x3e\xe2\xec\x21\xbf\x02\xff\xff\xff"
"\x0e\xf0\x0e\xbf\xa6\x21\x54\xbf\x1d\x44\x24\xbd\x02\xff\xff\xff"
"\x21\xce\x0a\xbf\xbe\x17\x57\xbf\x09\xea\x08\xbc\x02\xff\xff\xff"
"\xfe\x07\x79\xbf\xb8\xd0\x34\x3e\xba\xa9\x19\x3e\x02\xff\xff\xff"
"\x54\xa5\x70\xbf\x60\xa9\x7b\xbd\x35\xca\xab\x3e\x02\xff\xff\xff"
"\xda\xe9\x26\xbf\x81\x28\x32\xbf\x14\x16\x9a\x3e\x02\xff\xff\xff"
"\x18\xf4\xdf\xbe\xdb\xe2\x5d\x3f\x07\x6a\x75\x3e\x02\xff\xff\xff"
"\x22\xec\x6d\x3e\x9b\x3d\x65\x3f\x8b\x65\xc2\x3e\x02\xff\xff\xff"
"\xde\x3b\x04\x3f\x28\x90\x92\x3d\xfb\x6f\x5a\x3f\x02\xff\xff\xff"
"\x0d\x24\x78\x3f\xc7\x19\xb1\x3b\x1e\xb6\x7b\x3e\x02\xff\xff\xff"
"\x32\xb1\x69\x3f\xc7\xae\xbb\x3e\x35\x11\x38\xbe\x02\xff\xff\xff"
"\x55\xd9\x46\x3f\xe9\x23\x90\xbe\xcb\x39\x10\xbf\x02\xff\xff\xff"
"\x52\x8b\xae\x3e\xad\x98\x2e\x3f\x53\xa3\x25\xbf\x02\xff\xff\xff"
"\xdf\x4e\x49\xbe\x49\x8b\xb1\xbe\xa2\xc8\x6a\xbf\x02\xff\xff\xff"
"\xe5\xb6\xaa\xbe\x2d\x94\x61\x3f\xb7\xa2\xab\xbe\x02\xff\xff\xff"
"\x5b\x63\xc3\xbe\x7d\x56\xef\xbe\xcb\x22\x4c\x3f\x02\xff\xff\xff"
"\x56\x2c\x2f\x3e\xe0\x5a\xc1\x3e\xd5\xf6\x68\x3f\x02\xff\xff\xff"
"\x30\x78\x2a\x3f\xac\xc5\x02\xbf\x86\x31\x0b\x3f\x02\xff\xff\xff"
"\x67\x81\x1a\x3f\x0e\x94\x46\x3f\x8f\xef\x3c\x3e\x02\xff\xff\xff"
"\xdf\xbc\x92\xbd\x18\xd1\xef\xbc\xaf\x3b\x7f\xbf\x03\xff\xff\xff"
"\xe0\x05\x63\xbf\xb9\x5b\xe8\xbc\xbc\x2a\xec\xbe\x03\xff\xff\xff"
"\xdd\xc0\x16\xbf\xd4\x1e\x4e\xbf\xb0\x06\x90\xbd\x03\xff\xff\xff"
"\x12\x29\x06\xbf\x06\x06\x5a\xbf\x09\xb3\xe0\x3b\x03\xff\xff\xff"
"\x90\x1a\x67\xbe\xc6\x3e\xb3\x3e\x9b\xbc\x68\x3f\x03\xff\xff\xff"
"\x7f\x3e\x1d\x3e\x26\x64\xc8\xbe\x7e\x46\x68\x3f\x03\xff\xff\xff"
"\xd6\x7d\x53\x3f\xf1\x22\xd0\x3e\x80\xc5\xc7\x3e\x03\xff\xff\xff"
"\x4e\x3d\x32\x3f\x52\xb1\x35\x3f\xe7\xc4\xdb\x3d\x03\xff\xff\xff"
"\xfc\xde\x29\x3f\x43\x4f\x3e\x3f\x20\x03\xac\x3d\x03\xff\xff\xff"
"\x9c\x72\x5a\x3f\xb1\xe0\x04\xbf\x52\x67\x4a\xbd\x03\xff\xff\xff"
"\xf7\x48\x9d\x3e\x20\xa7\x3a\xbf\xcb\x90\x1c\xbf\x03\xff\xff\xff"
"\x2d\x63\xcb\xbd\xfb\xb1\xbf\x3e\xf0\x03\x6c\xbf\x03\xff\xff\xff"
"\x61\x0e\x45\xbf\x1a\xe8\xe6\xbe\x29\x4e\xe7\xbe\x03\xff\xff\xff"
"\xfe\xe0\x2a\xbf\xef\x51\x38\xbf\x1a\x6f\x42\xbe\x03\xff\xff\xff"
"\xbd\xb2\x8f\x3d\xf2\xf3\x79\x3f\xec\x49\x51\x3e\x03\xff\xff\xff"
"\x5b\x06\x14\x3f\x94\x09\xe9\x3e\x4d\x58\x2d\x3f\x03\xff\xff\xff"
"\x92\x98\x0c\x3f\x43\xaa\xd5\xbe\x5a\x5a\x39\x3f\x03\xff\xff\xff"
"\x6c\x6e\x1e\x3f\x77\x18\x45\xbf\xbd\x74\x1f\xbe\x03\xff\xff\xff"
"\x29\x45\x9c\x3e\x22\x5b\x31\xbf\xfe\x42\x27\xbf\x03\xff\xff\xff"
"\x27\xd4\x95\xbc\x90\x44\x4b\xbd\x90\xa4\x7f\xbf\x03\xff\xff\xff"
"\x86\xce\x7d\xbd\x03\x67\x3d\xbe\x61\x15\x7b\xbf\x03\xff\xff\xff"
"\x3b\xd3\x07\xbf\x04\x3d\x06\xbf\x64\x7e\x2a\xbf\x03\xff\xff\xff"
"\x54\x20\x01\xbf\xad\x47\x4e\x3f\xd6\xe2\x9e\xbe\x03\xff\xff\xff"
"\x3b\xf2\xb7\xbe\xda\xf0\xa4\xbe\x0b\x39\x60\x3f\x03\xff\xff\xff"
"\xa8\x6d\x92\xbe\x2b\xfa\xba\xbe\x79\xcb\x62\x3f\x03\xff\xff\xff"
"\x35\x18\xf3\x3b\xcb\x91\x26\xbf\xbe\x63\x42\x3f\x03\xff\xff\xff"
"\x7b\xb8\xc2\x3e\xbd\xd0\xe8\x3e\xff\x2b\x4e\x3f\x03\xff\xff\xff"
"\xdd\xde\x9e\x3e\xfb\xf3\x16\xbf\x29\xe4\x3e\x3f\x03\xff\xff\xff"
"\x89\xfe\x7a\x3e\xba\x03\x6f\x3f\x79\xbc\x85\x3e\x03\xff\xff\xff"
"\xb4\x04\x3a\x3f\xb9\x22\xf4\x3e\xd9\x3d\xfd\x3e\x03\xff\xff\xff"
"\x82\xdb\x78\x3f\x08\x9d\xa9\xbc\x3b\x40\x6f\x3e\x03\xff\xff\xff"
"\x12\xca\x94\xbe\x21\xe1\x54\xbe\x0e\x1a\x6f\xbf\x03\xff\xff\xff"
"\x4d\x5d\x93\xbe\xc0\x92\x61\x3f\x50\x15\xc0\xbe\x03\xff\xff\xff"
"\x83\xaa\x61\xbf\x20\x25\x6a\xbd\x9d\xf8\xef\xbe\x03\xff\xff\xff"
"\x3b\xd7\x6d\xbf\xd4\x85\x5d\xbe\x5e\xa1\x99\xbe\x03\xff\xff\xff"
"\x57\xb8\x51\xbf\x6e\xa2\x12\xbf\x76\x52\xe6\xbc\x03\xff\xff\xff"
"\xb1\xbb\x08\xbf\xff\xb9\x57\xbf\x3f\x1f\x8b\x3d\x03\xff\xff\xff"
"\xc6\x92\xf7\xbe\x8e\xb1\x50\xbf\x17\x38\xa3\x3e\x03\xff\xff\xff"
"\x3d\x26\x17\xbf\xcb\x27\xca\x3e\x43\x34\x34\x3f\x03\xff\xff\xff"
"\x98\xf3\x4a\xbe\xde\xd8\x73\x3f\x8e\xac\x6c\x3e\x03\xff\xff\xff"
"\xc3\x27\xd1\xbe\xca\x80\x08\xbf\x14\xa6\x3d\x3f\x03\xff\xff\xff"
"\xae\x5f\xf7\x3e\x44\x50\xb9\xbe\xe2\x16\x4c\x3f\x03\xff\xff\xff"
"\x8f\xd3\x6e\x3f\x9e\xa1\xb7\x3e\x63\x8f\x03\xbd\x03\xff\xff\xff"
"\xd7\xc8\x78\x3f\xcd\x7b\x26\xbe\x7a\xcc\x2e\xbe\x03\xff\xff\xff"
"\x36\x34\x49\x3f\xc6\x2f\x11\x3f\xc9\x26\x7c\xbe\x03\xff\xff\xff"
"\x13\xa4\x57\x3e\xfa\x32\x31\x3f\xaf\xb8\x30\xbf\x03\xff\xff\xff"
"\x69\xf5\x1d\xbf\xd8\x93\xb9\xbe\x42\xd1\x32\xbf\x03\xff\xff\xff"
"\x2e\x84\x00\xbf\x48\xe2\x5c\xbf\xd6\xc2\x72\x3d\x03\xff\xff\xff"
"\x58\xef\xee\x3e\xcf\xe1\x54\xbe\x3e\x12\x5c\x3f\x03\xff\xff\xff"
"\x00\xf8\x21\x3f\xc8\x4b\x1a\x3f\xae\xf5\xf8\x3e\x03\xff\xff\xff"
"\xef\x2a\x4a\x3f\x40\x4c\x37\xbd\x4a\xa1\x1c\xbf\x03\xff\xff\xff"
"\xd1\x90\x46\xbd\x9c\x2b\x69\xbe\x94\xf7\x78\xbf\x03\xff\xff\xff"
"\xbf\x92\x05\xbf\x5c\x95\x00\xbf\x57\x86\x30\xbf\x03\xff\xff\xff"
"\xad\xfc\x44\xbf\x52\xf0\x1f\xbe\x03\x89\x1e\xbf\x03\xff\xff\xff"
"\x17\x38\x34\xbf\x3e\xc1\x1d\xbf\xc3\xc8\xb4\xbe\x03\xff\xff\xff"
"\x95\xa0\x1e\xbf\xde\xcc\x46\xbf\x01\x85\xe9\x3d\x03\xff\xff\xff"
"\x0d\xa3\x02\xbf\x0f\x74\x4f\xbf\x2e\x6c\x93\x3e\x03\xff\xff\xff"
"\xf5\x89\xe8\xbe\xcb\x46\x7c\xbd\x41\x87\x63\x3f\x03\xff\xff\xff"
"\xcb\xee\x7e\xbe\xa8\x2f\x41\x3f\x56\x6a\x1b\x3f\x03\xff\xff\xff"
"\x57\x5f\x7d\x3d\xa2\x30\x24\xbd\xfb\x4d\x7f\x3f\x03\xff\xff\xff"
"\xf7\xbf\x27\x3e\x2d\x74\xff\x3c\x9c\x6a\x7c\x3f\x03\xff\xff\xff"
"\x3b\x9d\xcd\x3e\x4e\x9c\x19\x3e\xc6\x48\x67\x3f\x03\xff\xff\xff"
"\x61\x1f\x02\x3f\xa0\x6c\x4f\x3f\x76\x64\x95\xbe\x03\xff\xff\xff"
"\xc8\x31\x0f\x3f\xa9\xec\x83\x3e\xb0\xb1\x49\xbf\x03\xff\xff\xff"
"\xc8\x31\x0f\x3f\xa9\xec\x83\x3e\xb0\xb1\x49\xbf\x03\xff\xff\xff"
"\x94\xc0\xe3\x3e\x31\xfc\x0b\x3f\x65\x95\x35\xbf\x03\xff\xff\xff"
"\xa2\xe0\x2b\x3e\x41\x99\x67\x3f\x39\x85\xc8\xbe\x03\xff\xff\xff"
"\x4d\x4e\x1e\xbe\xca\xa3\x81\x3e\xd8\x79\x74\xbf\x03\xff\xff\xff"
"\x36\xc9\xdc\xbe\xd5\xca\x4a\xbf\xc7\x22\xdd\xbe\x03\xff\xff\xff"
"\xc9\x6f\x7b\xbf\x25\x41\x28\x3e\x8f\xeb\xba\xbd\x03\xff\xff\xff"
"\xca\x87\x66\xbf\xce\x82\xdb\x3e\xd1\xa5\x94\xbd\x03\xff\xff\xff"
"\xd1\x04\xaa\xbe\xed\xef\x09\x3f\x4c\x33\x46\x3f\x03\xff\xff\xff"
"\x29\xf6\x5e\xbe\x32\xac\x47\xbf\x50\x33\x16\x3f\x03\xff\xff\xff"
"\x76\x55\x74\xbe\x09\xaf\x09\xbd\x2d\x75\x78\x3f\x03\xff\xff\xff"
"\xd5\x36\x25\x3f\x8b\x45\x43\x3f\x18\x6c\x27\x3d\x03\xff\xff\xff"
"\xab\xb3\x79\x3e\xd7\x2c\x23\x3e\x0e\xe6\x74\xbf\x04\xff\xff\xff"
"\x36\xdf\xa8\xbe\x1f\x29\x17\x3e\xd3\xb3\x6e\xbf\x04\xff\xff\xff"
"\xa4\xb1\xb3\xbe\xd0\xc9\x45\xbf\xa3\x70\x07\xbf\x04\xff\xff\xff"
"\xa4\x08\x01\xbf\x4e\x88\x2a\xbf\x6a\xbb\x0c\xbf\x04\xff\xff\xff"
"\x30\xe6\xed\xbe\x08\xf8\x58\xbf\x03\x55\x83\xbe\x04\xff\xff\xff"
"\x21\x88\x72\xbf\xf5\x18\x68\x3b\x76\xde\xa3\xbe\x04\xff\xff\xff"
"\xc5\x81\x17\xbf\x33\x7e\x11\x3d\x43\x27\x4e\x3f\x04\xff\xff\xff"
"\x70\xda\x57\x3f\x9d\x3c\x41\x3e\xc8\xe0\x00\x3f\x04\xff\xff\xff"
"\x73\x8b\x43\x3f\xfb\xfe\x1f\x3f\x3e\xe8\x24\x3e\x04\xff\xff\xff"
"\xf4\x91\x73\x3f\xbd\xf0\x9a\xbe\x37\x96\x66\xbd\x04\xff\xff\xff"
"\xfd\x5d\x97\xbe\x5d\xfe\x2b\x3d\x40\x52\x74\xbf\x04\xff\xff\xff"
"\xbf\x92\x05\xbf\x5c\x95\x00\xbf\x57\x86\x30\xbf\x04\xff\xff\xff"
"\xe0\x03\x0e\xbf\x82\x8f\xf8\xbe\x3e\xfb\x2c\xbf\x04\xff\xff\xff"
"\x17\x1c\x03\xbf\x45\x8e\x47\x3f\x81\xa6\xb8\xbe\x04\xff\xff\xff"
"\xdc\x7d\x13\xbf\x89\xa1\x50\xbf\x93\xd3\x7f\xbd\x04\xff\xff\xff"
"\xe4\xcf\x61\xbf\xbf\x60\xe4\x3e\x67\x35\x1b\xbe\x04\xff\xff\xff"
"\x67\x16\x41\xbf\xdb\xa7\xc8\x3e\x96\xdc\x06\x3f\x04\xff\xff\xff"
"\x1d\x50\x80\xbe\x5c\x58\x70\xbe\xe2\x6f\x70\x3f\x04\xff\xff\xff"
"\xd8\xda\x3b\x3f\xd7\xb8\xcd\xbe\x66\x3c\x0c\x3f\x04\xff\xff\xff"
"\x05\x28\x45\x3f\x16\xa8\x0c\xbf\x75\xe7\xa5\x3e\x04\xff\xff\xff"
"\x09\xfe\x7e\x3f\xe3\xe0\xc1\xbc\x60\xe1\xae\xbd\x04\xff\xff\xff"
"\x2d\x1c\xbb\x3e\x56\x2d\x6e\xbf\x94\x96\xee\x3c\x04\xff\xff\xff"
"\x9a\x4b\x4a\x3f\x81\xb5\xd7\x3e\xdf\xdb\xe3\xbe\x04\xff\xff\xff"
"\x4f\xbd\x10\x3f\x14\x53\xc7\x3e\xee\x27\x3a\xbf\x04\xff\xff\xff"
"\x7e\x62\xf8\x3d\xa8\x7f\x6e\xbf\x03\x66\xaf\xbe\x04\xff\xff\xff"
"\x5f\x26\x23\x3e\x6e\x1c\x17\xbf\x74\x94\x4a\xbf\x04\xff\xff\xff"
"\xa6\xd3\xd7\xbd\x15\x3f\x61\xbf\x6e\x42\xed\xbe\x04\xff\xff\xff"
"\x3f\x55\x3d\xbf\x49\x35\x17\xbf\x14\x3b\xa5\xbe\x04\xff\xff\xff"
"\x9b\x4a\x95\xbe\xee\xf2\x4b\xbf\x69\x88\x07\x3f\x04\xff\xff\xff"
"\x99\x20\x92\x3e\x68\x74\x70\x3f\xab\x22\x43\x3e\x04\xff\xff\xff"
"\x6d\xec\xe0\x3e\x83\x44\x43\xbf\x6d\xf9\xf2\x3e\x04\xff\xff\xff"
"\x2d\x67\xe8\x3e\xfd\xe9\x5a\x3f\x88\x39\x80\x3e\x04\xff\xff\xff"
"\x43\xc2\x4f\x3f\x4c\x7f\x35\xbe\x5e\x86\x0e\x3f\x04\xff\xff\xff"
"\x43\xa2\x4c\x3f\x78\x62\xb8\xbe\x84\x43\xf6\x3e\x04\xff\xff\xff"
"\xf8\x05\x3e\x3f\xa0\xda\x29\x3f\xe2\x25\xc0\x3d\x04\xff\xff\xff"
"\xaf\xb3\x4d\x3f\xf6\x03\x0a\x3f\x14\x38\x81\xbe\x04\xff\xff\xff"
"\x7b\x5c\x9c\x3e\x49\x39\x60\x3f\xf0\x49\xbf\xbe\x04\xff\xff\xff"
"\xa0\x83\x7f\xbd\x28\x27\x0d\x3f\xe1\xf8\x54\xbf\x04\xff\xff\xff"
"\xee\xce\xf1\xbe\x3a\xa6\x5f\xbf\x48\xf3\xef\xbd\x04\xff\xff\xff"
"\x09\xe8\x34\xbf\x27\xe9\x01\xbe\x68\x32\x32\x3f\x04\xff\xff\xff"
"\x62\xa3\xc4\xbe\x09\x1b\xe2\x3e\x35\x94\x4f\x3f\x04\xff\xff\xff"
"\x25\x49\x08\x3d\x77\xdb\x2e\x3f\x5a\xc9\x3a\x3f\x04\xff\xff\xff"
"\x33\xfa\x08\x3f\x39\x0d\xd0\xbe\x2d\x9c\x3d\x3f\x04\xff\xff\xff"
"\x4c\xf3\x20\x3f\xe2\xc3\x00\x3f\x3d\xd3\x17\x3f\x04\xff\xff\xff"
"\x02\xa7\x7d\x3f\x60\x35\xff\x3d\xd2\xbe\x55\x3d\x04\xff\xff\xff"
"\x7b\x35\x2f\x3f\x98\x38\x3a\x3f\x18\x34\x4a\xbd\x04\xff\xff\xff"
"\x5d\x80\x3a\x3f\xb3\x77\x17\x3f\xe8\xc1\xb0\xbe\x04\xff\xff\xff"
"\x37\x7b\x06\x3f\xbe\xec\xf5\x3e\x09\xcf\x33\xbf\x04\xff\xff\xff"
"\x11\x15\x2b\xbe\x88\x3b\x18\xbe\x6a\x84\x79\xbf\x04\xff\xff\xff"
"\x63\xe7\xf6\xbe\xf8\x99\x1e\xbf\xc2\x8f\x1e\xbf\x04\xff\xff\xff"
"\x1e\x60\x6a\xbf\x02\x38\xcb\x3e\x88\xbd\x85\x3d\x04\xff\xff\xff"
"\x4a\x1b\xeb\xbe\xdd\xf9\x52\xbf\x99\xc8\xa9\x3e\x04\xff\xff\xff"
"\x67\xab\xd2\xbe\x5c\x76\x5b\xbf\x1a\x75\x9e\x3e\x04\xff\xff\xff"
"\xaf\xd9\xeb\x3d\x12\x59\x26\xbf\x9e\x57\x40\x3f\x04\xff\xff\xff"
"\x3c\x8a\x27\x3f\x22\x4c\x1d\x3f\x77\x9a\xe1\x3e\x04\xff\xff\xff"
"\x67\x26\x7f\x3f\x19\x75\xee\x3c\x30\xa9\x9b\x3d\x04\xff\xff\xff"
"\xb0\xda\xc8\x3e\x51\x41\x6b\xbf\x81\xa3\x24\xbd\x04\xff\xff\xff"
"\xb3\x04\x28\x3f\x45\x45\x30\x3f\x4b\xeb\x9d\xbe\x04\xff\xff\xff"
"\x6a\x5a\x96\x3e\x3a\xcb\x67\x3f\x30\xed\x9c\xbe\x04\xff\xff\xff"
"\x95\x4b\x90\x3e\x9b\xb7\x4a\x3f\x58\xb2\x0a\xbf\x04\xff\xff\xff"
"\x48\x91\x20\x3d\x58\xdc\xea\x3e\xcd\x41\x63\xbf\x04\xff\xff\xff"
"\x70\x96\x9c\xbe\x9e\x78\xee\xbe\x1a\x94\x54\xbf\x04\xff\xff\xff"
"\x00\xf3\x01\xbf\x1b\xdf\x12\x3e\x10\x7d\x59\xbf\x04\xff\xff\xff"
"\x0b\x7c\xd5\xbe\xb9\x61\x58\x3f\xf8\x21\xab\x3e\x04\xff\xff\xff"
"\x52\xee\xf6\xbe\x77\xd9\x0c\x3f\xed\x82\x2e\x3f\x04\xff\xff\xff"
"\x66\x29\x05\xbf\xf2\xb3\x6b\xbe\x2e\x8d\x52\x3f\x04\xff\xff\xff"
"\xd3\xcf\xd8\xbe\x0b\x65\x01\xbf\x1f\x76\x40\x3f\x04\xff\xff\xff"
"\xdc\x48\xb2\xbe\xe0\x15\x1f\xbf\xd1\xab\x33\x3f\x04\xff\xff\xff"
"\x59\x3b\x82\x3c\x54\x26\x79\x3c\x6b\xf0\x7f\x3f\x04\xff\xff\xff"
"\xcb\x22\x9d\x3d\x28\xd8\xc0\xbe\x46\x55\x6c\x3f\x04\xff\xff\xff"
"\xa2\xec\x11\x3f\x32\x47\x4e\x3f\x0e\x85\x24\xbe\x04\xff\xff\xff"
"\xbd\x96\x6c\x3f\xc9\xb8\xb6\xbe\x30\x69\x0b\xbe\x04\xff\xff\xff"
"\xea\xe9\x11\x3d\x5c\x13\x2a\xbf\x72\x1f\x3f\xbf\x04\xff\xff\xff"
"\xec\x60\x47\xbd\x2c\x6c\x73\x3f\xb4\x8b\x9c\xbe\x04\xff\xff\xff"
"\x8a\x86\x30\xbf\xd7\x95\x15\x3f\x64\x14\xdb\xbe\x04\xff\xff\xff"
"\xa8\xca\x0f\xbf\x26\xe1\xdb\x3e\xef\x08\x35\x3f\x04\xff\xff\xff"
"\x93\xfa\x84\xbe\xc5\x96\x9b\xbe\xfd\xa7\x6a\x3f\x04\xff\xff\xff"
"\x5f\xc1\x17\x3f\x5f\x7c\x3f\x3f\x97\xd6\x98\x3e\x04\xff\xff\xff"
"\x32\xe2\x7b\x3f\xaf\x14\x30\x3e\x40\xa0\x45\xbd\x04\xff\xff\xff"
"\x52\xe8\x04\x3e\x48\x3c\x6d\xbf\xd0\x93\xb4\xbe\x04\xff\xff\xff"
"\xe4\xf9\x1e\xbf\xd9\x13\x1a\x3f\x79\x8a\x00\xbf\x04\xff\xff\xff"
"\xc0\x43\x87\xbe\x94\x27\x6e\x3f\x6a\x4d\x82\xbe\x04\xff\xff\xff"
"\x8e\xa8\x30\xbf\x62\x40\xea\xbe\x45\x8f\x0f\x3f\x04\xff\xff\xff"
"\x30\x6c\x05\xbf\xb3\x44\x21\xbf\xae\x66\x13\x3f\x04\xff\xff\xff"
"\x74\x9f\xaa\x3e\xab\x2d\x71\x3f\x72\x08\x19\x3d\x04\xff\xff\xff"
"\xaf\xe9\xe1\xbc\xdc\x62\x61\x3e\x80\x9f\x79\xbf\x05\xff\xff\xff"
"\xea\xf2\x63\xbf\x7c\x24\x07\xbc\x8a\xfc\xe8\x3e\x05\xff\xff\xff"
"\xda\x27\xe4\xbd\x7e\xe1\x90\x3e\x5c\xe0\x73\x3f\x05\xff\xff\xff"
"\x22\x75\xf9\x3d\xbe\x1c\x50\xbe\x22\xb6\x78\xbf\x05\xff\xff\xff"
"\xab\x4e\x66\xbe\x75\x25\x58\xbf\x80\x04\xf9\xbe\x05\xff\xff\xff"
"\x62\x0e\x24\xbf\x41\x10\x26\xbc\xce\x81\x44\xbf\x05\xff\xff\xff"
"\xc8\x62\x44\xbf\x21\x38\x24\xbf\x4f\x3d\x55\xbb\x05\xff\xff\xff"
"\xb8\x8f\xa9\xbc\x51\x44\x85\xbe\xbe\x1e\x77\x3f\x05\xff\xff\xff"
"\x83\xbe\xc8\x3e\xdb\xa9\x2e\x3f\x84\xf8\x1d\x3f\x05\xff\xff\xff"
"\xd0\xa8\x3e\x3e\x18\x1b\x7b\x3f\xb2\x27\x68\x3d\x05\xff\xff\xff"
"\xcf\x6d\xf3\x3e\xdd\x3c\x4a\xbf\x80\x35\xc6\x3e\x05\xff\xff\xff"
"\x3f\x89\x7a\x3f\xa7\x87\x4e\xbe\x41\x22\x21\x3d\x05\xff\xff\xff"
"\xba\x1d\x6b\x3f\x0c\x26\xca\xbe\xa2\xd3\xca\x3c\x05\xff\xff\xff"
"\x18\x74\x84\xbe\x21\xdd\x58\xbf\x95\xa9\xed\xbe\x05\xff\xff\xff"
"\x74\x05\xec\xbe\xbb\xa9\xf6\xbe\x46\xc9\x3e\xbf\x05\xff\xff\xff"
"\x61\xa6\x24\xbf\x52\xac\xbe\xbe\x4b\x48\x2b\xbf\x05\xff\xff\xff"
"\x01\x6b\x11\xbf\x41\x72\x17\x3e\x63\x42\x4f\x3f\x05\xff\xff\xff"
"\x7b\x96\xfe\x3e\xa2\x12\x51\xbf\x99\xf2\x95\x3e\x05\xff\xff\xff"
"\xe4\x54\x6f\x3f\xc8\xe1\xb3\x3e\xfd\x9f\x4e\xbd\x05\xff\xff\xff"
"\x2c\x3d\x79\x3f\x38\xdd\x2a\xbe\x74\x8c\x1f\xbe\x05\xff\xff\xff"
"\x8c\xe1\x02\x3f\x8d\x94\x49\xbf\x6f\x51\xb0\xbe\x05\xff\xff\xff"
"\xab\x53\x43\xbd\x3f\x11\x82\x3d\x3f\x31\x7f\xbf\x05\xff\xff\xff"
"\x82\x98\xed\xbd\x79\xbc\x81\x3e\xd2\xdb\x75\xbf\x05\xff\xff\xff"
"\x9c\xc6\xbe\xbe\x53\x80\xe0\x3e\x92\x5f\x51\xbf\x05\xff\xff\xff"
"\xe6\x41\xe4\xbe\x6f\x65\x49\xbf\xac\xa3\xda\xbe\x05\xff\xff\xff"
"\x66\xce\x53\xbf\xf3\x91\x0e\x3f\x86\x31\x95\xbd\x05\xff\xff\xff"
"\x45\x4d\x10\xbf\xd2\xa6\xbc\xbe\x23\x40\x3d\x3f\x05\xff\xff\xff"
"\x53\xf1\x5f\x3d\xf7\x93\x07\x3f\x8f\xb3\x58\x3f\x05\xff\xff\xff"
"\x14\x7e\x23\x3e\x2c\x1f\x15\x3f\x13\x08\x4c\x3f\x05\xff\xff\xff"
"\xdf\x7e\x97\x3e\xe5\x94\x73\x3f\x7e\xd6\xac\xbd\x05\xff\xff\xff"
"\x87\x3f\x51\x3f\x8a\x56\x12\xbf\xe5\xb5\x92\x3d\x05\xff\xff\xff"
"\xfd\x65\x5b\x3f\xac\xd0\xa2\x3e\x05\x96\xcf\xbe\x05\xff\xff\xff"
"\xfa\xc7\xd5\x3e\x66\x3a\x3a\xbf\x54\x64\x0b\xbf\x05\xff\xff\xff"
"\xbf\x61\xc6\x3e\x21\xbc\xa3\xbe\x60\x59\x5d\xbf\x05\xff\xff\xff"
"\x33\x59\x37\xbe\xab\xd0\x26\x3e\x1f\x63\x78\xbf\x05\xff\xff\xff"
"\x11\x14\x6b\xbf\x03\x1c\xc1\x3e\x77\xb8\xf6\xbd\x05\xff\xff\xff"
"\x17\x53\x12\xbf\x62\xce\x0c\xbf\x83\xe1\x1b\x3f\x05\xff\xff\xff"
"\xac\x85\xed\xbe\xa0\x84\x32\xbf\x70\xde\x0b\x3f\x05\xff\xff\xff"
"\x9b\x76\x75\xbe\xac\x03\x55\xbe\xc2\xc3\x72\x3f\x05\xff\xff\xff"
"\x04\x1f\x46\x3f\xa7\x99\x6c\xbd\x95\x72\x21\x3f\x05\xff\xff\xff"
"\x82\xdb\x78\x3f\x08\x9d\xa9\xbc\x3b\x40\x6f\x3e\x05\xff\xff\xff"
"\x1b\x20\x6d\x3f\xb4\x33\x50\x3d\x7c\x2f\xbf\xbe\x05\xff\xff\xff"
"\x7a\x7b\x8e\x3e\xa4\x30\x7a\xbe\xf0\xcc\x6d\xbf\x05\xff\xff\xff"
"\x31\x6c\xa5\xbe\xc2\xff\xc7\xbe\xb5\xab\x5c\xbf\x05\xff\xff\xff"
"\xef\x91\x1d\xbf\x2e\x11\xaf\x3e\x07\xc9\x35\x3f\x05\xff\xff\xff"
"\x6a\x7f\xca\xbe\x23\x12\x76\x3e\x11\xf0\x62\x3f\x05\xff\xff\xff"
"\x8e\x78\x20\x3f\xc1\x53\x10\x3f\x7b\xad\x09\x3f\x05\xff\xff\xff"
"\x34\x3a\x41\x3f\x8a\xdb\x93\x3e\x0a\xc7\x16\x3f\x05\xff\xff\xff"
"\x51\x79\x3a\x3f\x07\x96\x14\x3f\x68\x65\xba\x3e\x05\xff\xff\xff"
"\x16\xd6\x20\x3f\xa3\x42\x46\x3f\x37\xf6\x97\x3d\x05\xff\xff\xff"
"\x64\xbc\x67\x3f\x09\x1c\xc6\xbe\x25\xe3\x33\xbe\x05\xff\xff\xff"
"\x0f\x03\x52\x3f\x89\x78\xcb\xba\x7b\x64\x12\xbf\x05\xff\xff\xff"
"\x75\xe0\xd5\x3d\x5d\x79\x69\xbf\x4e\x18\xcb\xbe\x05\xff\xff\xff"
"\x30\xc6\x85\xbe\x1a\xdd\x0e\xbc\x32\x19\x77\xbf\x05\xff\xff\xff"
"\x04\x9a\x0c\xbf\x0b\x78\x2d\xbf\xc7\x68\xfa\xbe\x05\xff\xff\xff"
"\xf4\x84\x40\xbf\x02\xfb\xc5\x3c\xbe\x9f\x28\xbf\x05\xff\xff\xff"
"\xa1\x13\xe8\x3d\xdb\xde\x62\xbf\x17\xff\xe5\x3e\x05\xff\xff\xff"
"\x24\xf4\xfd\x3e\x3f\x8f\xf2\x3e\xb1\x4a\x3a\x3f\x05\xff\xff\xff"
"\x40\x7e\xd3\x3d\xc1\xdd\x3e\xbf\xf7\x8b\x28\xbf\x05\xff\xff\xff"
"\x80\xcf\x01\x3d\x95\xd5\x9d\xbe\x16\x66\x73\xbf\x05\xff\xff\xff"
"\x04\x59\x05\xbf\x37\xc2\xde\xbe\xc1\x03\x3c\xbf\x05\xff\xff\xff"
"\x24\x21\x0c\xbf\x61\x59\x2b\xbf\x54\x9b\x00\xbf\x05\xff\xff\xff"
"\xd7\xd1\x15\xbf\xe9\xc5\x20\xbf\x8e\x4f\x03\xbf\x05\xff\xff\xff"
"\xad\x47\x24\xbf\x2c\x33\x2c\xbf\x1d\xa2\xbc\xbe\x05\xff\xff\xff"
"\xd1\x54\x10\xbf\x9f\x5c\x53\xbf\xea\x28\xb2\xbc\x05\xff\xff\xff"
"\xda\x83\x54\xbe\x96\xa9\x79\xbf\x48\x78\x9c\x3d\x05\xff\xff\xff"
"\xb0\xcc\x5e\xbe\xba\xdf\x84\xbe\x9d\xdf\x70\x3f\x05\xff\xff\xff"
"\x6e\x8e\x31\x3e\x48\x8f\xf0\x3d\xe2\x52\x7a\x3f\x05\xff\xff\xff"
"\x31\x65\x38\x3e\x57\xd0\x78\x3f\x32\x16\x1b\x3e\x05\xff\xff\xff"
"\x32\xab\x0b\x3f\x36\x72\x42\x3e\xc5\xf6\x50\x3f\x05\xff\xff\xff"
"\x60\x99\xfe\x3e\x02\x25\x20\xbf\x48\xe6\x19\xbf\x05\xff\xff\xff"
"\x4d\x94\x25\x3d\x0c\x4c\x67\xbd\x10\x62\x7f\xbf\x05\xff\xff\xff"
"\x26\xd7\x42\xbf\xff\x3f\xdb\x3e\x7e\x71\xf9\xbe\x05\xff\xff\xff"
"\x87\x22\x3b\xbf\xe7\xd5\x29\xbf\x7b\x95\x23\x3e\x05\xff\xff\xff"
"\x30\x3f\x0e\xbf\x58\x43\x16\x3f\x16\xbe\x16\x3f\x05\xff\xff\xff"
"\x19\xfe\x32\x3f\x69\xc1\x23\xbf\x13\x7c\xa3\x3e\x05\xff\xff\xff"
"\xac\x4e\x3c\x3f\x46\xc6\x2c\x3f\x3b\x3d\x70\x3d\x05\xff\xff\xff"
"\x24\x95\x4e\x3f\x57\xf5\x10\xbf\xd2\xe3\x2b\x3e\x05\xff\xff\xff"
"\xa9\x99\x15\x3f\x89\x39\xd7\xbe\x14\xb2\x31\xbf\x05\xff\xff\xff"
"\x9d\xae\xed\x3e\xa2\x54\xc4\x3e\xab\x64\x4c\xbf\x05\xff\xff\xff"
"\x91\x11\x51\x3e\x32\x63\x5b\x3f\x03\x45\xf2\xbe\x05\xff\xff\xff"
"\xdd\x28\xce\x3d\x03\xa1\x3c\x3f\xae\x25\x2b\xbf\x05\xff\xff\xff"
"\x5e\x61\xd4\x3d\x07\x64\xa1\xbd\x09\xd2\x7d\xbf\x05\xff\xff\xff"
"\xd2\xe4\xb9\xbe\x20\xf7\x1d\xbd\xc3\x53\x6e\xbf\x05\xff\xff\xff"
"\x25\xe3\x7f\xbf\x16\x57\xa5\xbc\x4a\x26\xb1\xbc\x05\xff\xff\xff"
"\xe0\xe4\x27\xbf\xea\x17\x3a\xbf\x77\x7f\x50\x3e\x05\xff\xff\xff"
"\xbd\x35\x46\xbe\xe1\x42\x28\xbf\xea\x76\x3a\x3f\x05\xff\xff\xff"
"\x82\xd2\x64\x3f\x9a\xf1\xde\xbe\x90\x1d\xdb\x3d\x05\xff\xff\xff"
"\xea\xde\x4a\xbe\x15\x67\x1e\x3e\x34\xc8\x77\xbf\x05\xff\xff\xff"
"\x4a\xc6\xb5\xbd\x10\x37\x4e\x3f\x13\xfc\x15\xbf\x05\xff\xff\xff"
"\xae\x40\x87\xbe\xd2\xc5\x1b\xbf\x64\x91\x3f\xbf\x05\xff\xff\xff"
"\x6c\xdf\x94\xbe\xca\x4d\x74\xbf\x60\x2d\x8d\xbd\x05\xff\xff\xff"
"\x50\x2c\xf7\xbe\x74\x3c\x44\x3f\xc8\xd4\xd8\xbe\x05\xff\xff\xff"
"\xb1\x24\x3f\xbf\x08\xd3\x14\x3f\x38\x8c\xa5\xbe\x05\xff\xff\xff"
"\x9d\x02\x70\xbf\x1a\x17\xb2\xbe\x6c\x65\xad\xbb\x05\xff\xff\xff"
"\x58\x4c\xc7\x3e\xc6\x88\x4d\x3f\xfe\x30\xe7\x3e\x05\xff\xff\xff"
"\x4f\xfb\xbc\x3e\x98\x94\x68\xbf\x29\x91\x48\x3e\x05\xff\xff\xff"
"\xfa\x78\xd5\x3d\x0d\x51\x7c\x3f\xa7\x48\x08\xbe\x06\xff\xff\xff"
"\x0c\x82\x23\x3f\x68\x77\x2b\x3f\x88\xe9\xc1\xbe\x06\xff\xff\xff"
"\x59\xb7\xdc\x3e\x83\x8c\x4e\xbf\xe0\xd5\xce\xbe\x06\xff\xff\xff"
"\x98\xf7\x1a\x3f\x36\x61\xb0\xbe\x11\xb2\x37\xbf\x06\xff\xff\xff"
"\x98\xe9\x0f\x3e\xb0\x05\x59\xbf\x1d\xef\x02\xbf\x06\xff\xff\xff"
"\x0b\xa9\x52\x3e\xe8\xa9\x86\x3d\x83\xf5\x79\xbf\x06\xff\xff\xff"
"\x7f\x09\x36\x3e\x0e\x50\x12\x3f\x7c\x14\x4d\xbf\x06\xff\xff\xff"
"\xdc\xb7\xf8\xbe\xec\x55\x26\x3f\x82\xad\x15\xbf\x06\xff\xff\xff"
"\x81\x57\x48\xbf\xe6\x6e\x38\x3e\x29\x8e\x18\xbf\x06\xff\xff\xff"
"\x46\xdd\x5a\xbf\xda\xef\xfd\xbe\x9b\x9b\x1b\xbe\x06\xff\xff\xff"
"\x0d\xe7\x00\xbe\x3e\x38\x7d\x3f\xfd\x85\x9b\x3d\x06\xff\xff\xff"
"\x62\x79\x4e\x3f\xc4\x92\x79\x3d\x32\x89\x16\xbf\x06\xff\xff\xff"
"\xce\x67\xa7\xbd\xa7\xb5\x25\x3f\xd5\x01\x42\xbf\x06\xff\xff\xff"
"\x06\xfe\x65\xbe\x67\xa0\x76\xbf\xf1\xf9\x15\x3e\x06\xff\xff\xff"
"\x0e\xe2\x2e\xbf\x50\x8d\x36\x3f\xd8\x56\x21\x3e\x06\xff\xff\xff"
"\xf9\x0f\x43\xbf\x11\x00\x0b\x3f\x89\xba\xb4\x3e\x06\xff\xff\xff"
"\x1a\xc5\x43\xbf\xb1\x15\x04\x3e\xbe\x9d\x21\x3f\x06\xff\xff\xff"
"\x53\x72\x42\xbe\x1a\x33\x48\x3f\x6a\xf7\x17\x3f\x06\xff\xff\xff"
"\x02\x5e\x93\x3d\x78\x4f\xfd\xbd\xcf\x5d\x7d\x3f\x06\xff\xff\xff"
"\x61\x74\x21\x3f\xb2\xdf\x87\x3d\x8c\xf0\x45\xbf\x06\xff\xff\xff"
"\xf9\xd0\xad\x3e\xbc\xd3\x42\xbf\x73\x83\x0d\xbf\x06\xff\xff\xff"
"\xb8\xa4\xcf\x3e\x10\xbe\xf1\xbe\x7f\x5e\x48\xbf\x06\xff\xff\xff"
"\x9d\x5f\x48\x3e\x5b\xe1\xda\x3d\x83\x8e\x79\xbf\x06\xff\xff\xff"
"\x9d\x4f\x45\x3d\x92\x65\x83\xbe\xbb\x1e\x77\xbf\x06\xff\xff\xff"
"\xee\x9d\xf8\xbe\x08\x46\x43\xbf\xf4\xa6\xda\xbe\x06\xff\xff\xff"
"\x76\x97\x3b\xbf\x71\xf8\x13\xbf\x57\xd9\xb7\x3e\x06\xff\xff\xff"
"\xd7\xb8\x1c\xbf\xcf\x2f\x34\xbf\xad\x77\xb8\x3e\x06\xff\xff\xff"
"\xff\x81\xc1\xbe\x98\x3c\x29\xbf\xee\xee\x25\x3f\x06\xff\xff\xff"
"\x62\x9f\x42\x3f\x7a\xdd\x9b\x3e\xd2\xea\x12\x3f\x06\xff\xff\xff"
"\x5b\x18\x68\x3f\x4f\x0d\x7d\xbe\xc2\x1a\xaf\x3e\x06\xff\xff\xff"
"\x14\xc1\x66\x3f\xfd\x83\x52\x3d\x52\x23\xdc\xbe\x06\xff\xff\xff"
"\x9c\xb3\x2f\x3f\x4c\x6e\x24\x3d\xd5\xe6\x39\xbf\x06\xff\xff\xff"
"\x81\x8d\xe2\x3e\x6a\x15\xdb\xbd\xab\xef\x63\xbf\x06\xff\xff\xff"
"\xb5\x1a\x75\x3e\xbc\x27\x90\x3e\x08\xe1\x6d\xbf\x06\xff\xff\xff"
"\x73\xaa\x34\x3e\xee\x0a\x92\x3e\x35\x2c\x71\xbf\x06\xff\xff\xff"
"\xed\xa9\xf7\xbe\x8d\x69\xdb\x3e\xbc\x5c\x43\xbf\x06\xff\xff\xff"
"\xff\xb1\x17\xbf\xcc\x25\x4e\xbf\x9f\x46\xa9\xbc\x06\xff\xff\xff"
"\x97\xdc\x58\xbf\x91\x47\x01\xbf\x90\x68\x29\x3e\x06\xff\xff\xff"
"\x89\x8c\x65\xbe\x2a\x01\x35\xbe\xfe\x58\x75\x3f\x06\xff\xff\xff"
"\xef\xc0\x50\x3f\x88\x1e\x8d\xbe\xd3\x4d\x02\x3f\x06\xff\xff\xff"
"\x44\x00\x31\x3f\xe7\x42\x36\xbf\x6e\x66\xfb\x3d\x06\xff\xff\xff"
"\xf4\x5b\x7f\x3f\x58\x44\x80\x3d\xb1\x1e\x06\xbd\x06\xff\xff\xff"
"\x3b\x08\x77\x3e\x3b\x3d\x31\x3f\xe9\x17\x2e\xbf\x06\xff\xff\xff"
"\x15\x08\x94\x3e\x66\xbe\x2b\x3d\x17\xd5\x74\xbf\x06\xff\xff\xff"
"\x91\x45\x7c\x3e\x0c\xa9\x4b\x3e\x86\xd4\x72\xbf\x06\xff\xff\xff"
"\x21\xb8\xb9\x3d\x24\xc6\x5d\x3f\x63\x81\xfb\xbe\x06\xff\xff\xff"
"\xae\x9f\xfd\xb8\x28\x51\xa8\xbe\xd2\xc5\x71\xbf\x06\xff\xff\xff"
"\x74\x6c\xb7\xbd\x9f\x3b\x7b\x3f\x25\x07\x2e\xbe\x06\xff\xff\xff"
"\xff\x2c\x37\xbb\x18\x5e\xcf\xbe\x27\x10\x6a\x3f\x06\xff\xff\xff"
"\x97\xfc\xac\x3c\x8b\x81\x0a\xbf\x6e\x3a\x57\x3f\x06\xff\xff\xff"
"\xd9\x78\xa6\x3e\x52\x03\x27\xbf\x8d\x41\x2f\x3f\x06\xff\xff\xff"
"\x45\x1b\xf6\x3e\x0f\xce\x5e\x3f\x4a\x6d\xdb\x3d\x06\xff\xff\xff"
"\x9f\x9d\xe3\x3e\xdb\xf3\x64\x3f\x31\x7e\x4d\x3d\x06\xff\xff\xff"
"\xb9\xf8\x51\x3f\x6b\xd0\x08\x3f\x08\xfe\x50\x3e\x06\xff\xff\xff"
"\x64\x91\x4c\x3f\xc0\xc2\x0c\xbf\xaa\xf9\x78\x3e\x06\xff\xff\xff"
"\x24\xcd\x41\x3f\x38\x75\x24\x3f\x7a\x0e\xf4\xbd\x06\xff\xff\xff"
"\x93\x98\xc8\x3e\x9a\xa8\x9d\x3e\x9f\xf3\x5d\xbf\x06\xff\xff\xff"
"\x24\xc8\x43\xbf\x52\x26\xe1\x3c\x70\xca\x24\xbf\x06\xff\xff\xff"
"\x6b\x53\x7f\xbf\x78\x80\x1c\x3d\x88\x50\x7c\x3d\x06\xff\xff\xff"
"\xb3\xb7\xae\x3e\xf4\xad\x28\x3f\xed\x9d\x2b\x3f\x06\xff\xff\xff"
"\x9a\x49\xd3\x3e\x5c\xa7\x5a\x3f\x76\x11\xa2\x3e\x06\xff\xff\xff"
"\xef\xa8\x45\x3f\x18\x7e\x0f\xbf\x3e\x51\x99\xbe\x06\xff\xff\xff"
"\x77\x1c\x4c\xbf\x34\xae\xc2\x3e\xb1\xfe\xef\xbe\x06\xff\xff\xff"
"\x9b\x7e\x6d\xbf\x93\x15\x52\xbe\x7d\xab\x9f\x3e\x06\xff\xff\xff"
"\xda\x3b\xb9\xbe\x46\x74\x31\x3f\xe5\x96\x1f\x3f\x06\xff\xff\xff"
"\x2d\x3b\x1e\x3e\x06\xae\xe0\xbe\xea\x9b\x62\xbf\x06\xff\xff\xff"
"\x77\x64\x02\xbf\x78\x83\xb8\x3d\x09\x18\x5b\xbf\x06\xff\xff\xff"
"\x2d\x44\x44\xbf\x9d\x35\x86\xbe\xc1\x0a\x16\xbf\x06\xff\xff\xff"
"\x8b\x56\xf5\xbe\x74\xa3\x5d\xbf\x06\xc9\x13\x3e\x06\xff\xff\xff"
"\xac\x48\xf9\xbd\xfd\xc3\x79\xbf\x9d\xe0\x3a\x3e\x06\xff\xff\xff"
"\x63\x40\xf2\xbe\xd0\xdf\x1a\xbf\x7e\xf2\x23\x3f\x06\xff\xff\xff"
"\xfd\x7e\x2e\xbd\x9a\xc1\x7e\x3f\x0e\xe5\xb5\x3d\x06\xff\xff\xff"
"\xc8\x58\xa1\xbe\xc1\xda\x7d\xbe\x9b\x85\x6a\x3f\x06\xff\xff\xff"
"\xb8\x03\x41\x3f\x65\xce\x73\x3c\xb0\x20\x28\x3f\x06\xff\xff\xff"
"\x8c\x89\x0b\x3f\x29\xcc\x43\x3f\x00\xd4\xaf\xbe\x06\xff\xff\xff"
"\x98\x3f\x44\x3f\x9a\xfb\x5c\xbd\xe5\xcd\x23\xbf\x06\xff\xff\xff"
"\x10\x16\x96\xbe\x7b\x96\xf2\xbe\x23\x96\x54\xbf\x06\xff\xff\xff"
"\xd5\x97\xd8\xbe\xbe\xf8\x5f\xbf\xbc\x7f\x71\xbe\x06\xff\xff\xff"
"\x6c\xeb\xf6\xbe\x31\xf9\x5b\xbf\x12\x9e\x2e\x3e\x06\xff\xff\xff"
"\x64\xb0\x23\xbf\x7b\xf4\x31\xbf\xeb\x36\xa8\x3e\x06\xff\xff\xff"
"\x57\x49\xdd\xbe\xa0\xcd\x44\xbf\x59\x5a\xf1\x3e\x06\xff\xff\xff"
"\x96\x12\xb5\xbe\x4b\x8e\x16\xbf\x86\x34\x3a\x3f\x06\xff\xff\xff"
"\x5d\xf8\x7a\xbc\xab\xa1\xf3\xbd\x21\x27\x7e\x3f\x06\xff\xff\xff"
"\xf0\x2f\x6a\x3e\xa6\x45\x28\x3f\x8a\xd4\x37\x3f\x06\xff\xff\xff"
"\x86\x63\x2a\xbf\x86\xa7\xcb\xbd\x37\x5b\x3d\xbf\x06\xff\xff\xff"
"\x57\x0f\xfe\xbe\x1c\xb5\x58\xbf\x59\x7e\x45\xbe\x06\xff\xff\xff"
"\xd0\xa5\x11\xbf\xdf\x6c\x23\xbf\xc2\xb8\x04\x3f\x06\xff\xff\xff"
"\xe9\xfe\xef\xbe\xab\x6e\xcd\xbe\xbf\x75\x49\x3f\x06\xff\xff\xff"
"\xa1\xb9\xbe\x3c\x23\x00\x33\x3e\x3a\xfd\x7b\x3f\x06\xff\xff\xff"
"\x4e\xd0\x61\x3f\xd0\x89\x98\x3e\x84\xd6\xba\x3e\x06\xff\xff\xff"
"\x6e\x18\x3d\x3f\xaf\x53\x0a\xbf\x48\x58\xce\x3e\x06\xff\xff\xff"
"\xe2\xa7\x05\x3f\x5e\xfa\x6a\x3e\xf6\x49\x52\xbf\x06\xff\xff\xff"
"\x41\xb7\xa8\xbd\x90\x55\x72\xbf\x64\x93\x9f\xbe\x06\xff\xff\xff"
"\x27\xab\x36\xbe\x3f\xf8\xd1\x3e\x50\xf9\x64\xbf\x06\xff\xff\xff"
"\xdc\x5c\x30\xbe\xce\x57\x4e\x3f\x18\xf7\x10\xbf\x06\xff\xff\xff"
"\xd6\x60\x91\xbe\x77\x50\x71\xbf\xab\xca\x33\xbe\x06\xff\xff\xff"
"\x71\xbb\x49\xbe\x7e\xa0\x7a\xbf\xe1\x38\x56\xbd\x06\xff\xff\xff"
"\xd5\x48\x1e\xbf\x83\xdb\x41\xbf\xac\x77\x57\xbe\x06\xff\xff\xff"
"\xba\x01\x05\xbf\x9b\xcb\x21\xbf\x06\x33\x13\x3f\x06\xff\xff\xff"
"\xaf\x25\x46\x3d\xdc\x64\x59\xbf\x05\x9f\x06\x3f\x06\xff\xff\xff"
"\x88\xfd\x4f\x3f\x34\x9a\xf9\x3e\xd3\xb6\xa3\xbe\x06\xff\xff\xff"
"\x36\x28\xee\x3e\xc5\x4a\x61\xbf\xa7\xfe\xc3\xbd\x06\xff\xff\xff"
"\x11\x51\xb1\x3e\xf0\x92\x29\x3f\x86\x0f\x2a\xbf\x06\xff\xff\xff"
"\xce\xb2\xa5\x3e\x69\x4f\x89\x3e\x57\x4a\x68\xbf\x06\xff\xff\xff"
"\x13\x76\x11\x3e\x1d\x95\x39\x3d\xad\x23\x7d\xbf\x06\xff\xff\xff"
"\xa5\xc7\xdc\x3c\xec\xde\xa6\xbe\xc4\xec\x71\xbf\x06\xff\xff\xff"
"\x8f\x0e\x1c\xbe\x63\x04\x27\xbf\x74\x0d\x3e\xbf\x06\xff\xff\xff"
"\x40\x2f\x46\xbf\x09\x08\x93\x3d\x0e\x00\x21\xbf\x06\xff\xff\xff"
"\x43\xbb\x52\xbf\xc0\xdb\x08\x3f\xbe\xe0\x43\x3e\x06\xff\xff\xff"
"\xb4\xfa\x18\xbf\x62\x9d\x13\xbf\x45\xa1\x0e\x3f\x06\xff\xff\xff"
"\x31\x46\x46\x3d\xad\x70\x7f\x3f\xed\xc4\x38\xbd\x06\xff\xff\xff"
"\x9c\x83\x52\x3e\x75\x39\xe1\x3e\x3a\xcc\x5f\x3f\x06\xff\xff\xff"
"\x65\x0f\xaf\x3d\x04\xae\x54\xbf\xfc\xcc\x0c\x3f\x06\xff\xff\xff"
"\x99\x98\x9d\x3e\x01\xb6\x2b\x3f\x47\xc0\x2c\x3f\x06\xff\xff\xff"
"\xac\x4a\xb7\x3e\x19\x24\xc6\x3e\xac\x89\x59\x3f\x06\xff\xff\xff"
"\xe7\xea\xce\x3e\xe2\x17\x2b\x3f\xfe\xdf\x1f\x3f\x06\xff\xff\xff"
"\x3f\x7a\x0d\x3f\x10\xf2\x21\x3d\x6a\x1d\x55\x3f\x06\xff\xff\xff"
"\xa3\x47\x30\x3f\x64\x8f\x38\x3f\xf1\xd5\x9f\x3d\x06\xff\xff\xff"
"\x53\xe8\xc8\x3e\x59\x3f\x6b\xbf\x21\x48\x23\xbd\x06\xff\xff\xff"
"\x08\x6e\x58\x3f\xff\x83\xd6\x3e\xb5\x95\xa9\xbe\x06\xff\xff\xff"
"\xa5\x4f\x3b\x3f\x54\xef\x4b\x3d\xec\x08\x2e\xbf\x06\xff\xff\xff"
"\x3f\x24\x60\xbe\x65\x5e\x25\xbf\x69\x37\x3b\xbf\x06\xff\xff\xff"
"\x24\xff\xc9\xbe\xcd\x1f\xbb\xbe\x28\xd4\x57\xbf\x06\xff\xff\xff"
"\xc6\xbd\x6c\xbf\x03\xb8\x3a\x3e\x25\x00\xab\xbe\x06\xff\xff\xff"
"\xe4\x62\x0d\x3f\x11\xab\xba\x3e\xb9\xec\x3f\x3f\x06\xff\xff\xff"
"\x0f\x9d\x0e\x3f\x10\x14\xb6\x3e\x70\x1e\x40\x3f\x06\xff\xff\xff"
"\xc7\x92\xfb\x3e\xf5\x1d\x08\xbf\x7d\x98\x30\x3f\x06\xff\xff\xff"
"\xd5\x99\x42\x3f\xc7\x26\x26\xbf\x81\x49\xf8\x3c\x06\xff\xff\xff"
"\x11\x56\x27\x3f\x4b\xc4\x15\x3f\x20\xcd\xf5\xbe\x07\xff\xff\xff"
"\x7a\xcf\x5b\x3e\x5e\xc7\x76\xbf\x0e\xda\x20\xbe\x07\xff\xff\xff"
"\xce\x4e\x01\x3f\xfd\x1d\x00\xbf\x69\x01\x34\xbf\x07\xff\xff\xff"
"\x8b\x47\x69\xbd\x01\x68\x76\x3d\xff\x1e\x7f\xbf\x07\xff\xff\xff"
"\x31\x2f\xb9\xbe\xb5\xb6\xd5\x3e\x9c\x69\x55\x3f\x07\xff\xff\xff"
"\x28\xdc\x4a\x3e\x5c\x96\x76\xbf\xd5\xdf\x39\xbe\x07\xff\xff\xff"
"\x84\x14\xef\x3e\xd4\x71\x38\xbf\x55\x40\x03\xbf\x07\xff\xff\xff"
"\x44\x7b\xed\x3e\x5c\xd5\xe4\x3d\x70\xfc\x60\xbf\x07\xff\xff\xff"
"\x23\x5c\xe4\xbe\xe2\x04\x42\xbf\x80\xc5\xf3\xbe\x07\xff\xff\xff"
"\x77\x1c\x74\xbe\x63\x1e\x78\xbf\x29\x9b\x7c\x3d\x07\xff\xff\xff"
"\x25\xce\x04\xbf\x91\xe5\x57\xbf\x5f\x89\x0f\x3e\x07\xff\xff\xff"
"\xaa\x0e\xd0\xbe\xde\x16\x65\xbf\x5e\xff\x3c\x3e\x07\xff\xff\xff"
"\xee\x2b\x61\xbe\x6c\x33\x5d\xbf\x7c\xd7\xe7\x3e\x07\xff\xff\xff"
"\x6e\xe6\xa7\x3e\xf8\x94\x0d\xbf\xc4\x11\x44\x3f\x07\xff\xff\xff"
"\xf3\xa8\xa4\x3e\x4c\xdd\x12\xbf\x1b\xd8\x40\x3f\x07\xff\xff\xff"
"\x48\x6e\x2f\x3f\x3a\xb2\xd9\xbe\x18\x5e\x17\x3f\x07\xff\xff\xff"
"\xe9\xda\x2b\x3f\x92\xb8\x33\xbf\x94\x62\x73\x3e\x07\xff\xff\xff"
"\xe5\xfb\xe5\x3e\x92\x53\x2e\x3f\xb9\x10\x14\xbf\x07\xff\xff\xff"
"\x7a\x39\x29\xbe\x93\x7e\x95\x3d\xe0\xc9\x7b\xbf\x07\xff\xff\xff"
"\x57\xfe\xd9\xbe\x1d\xd3\xad\xbe\x04\xb7\x56\xbf\x07\xff\xff\xff"
"\x77\x2b\x67\xbf\x56\x74\x81\xbe\x1b\xd7\xb1\xbe\x07\xff\xff\xff"
"\x3c\x80\x76\xbf\x3a\x74\x5b\x3e\x07\xf4\x27\x3e\x07\xff\xff\xff"
"\x0a\xc6\x10\x3f\xeb\x03\x50\xbf\x97\x94\x10\x3e\x07\xff\xff\xff"
"\x1a\x32\x1d\x3e\x51\x8e\x31\xbf\xc5\x2e\x34\x3f\x07\xff\xff\xff"
"\xfd\x92\xdc\x3e\x40\x7b\x1f\xbf\x04\x26\x27\x3f\x07\xff\xff\xff"
"\xee\x21\x57\x3f\xdc\xd1\xdc\x3e\x56\x17\xa8\x3e\x07\xff\xff\xff"
"\xe4\x0b\x49\x3f\xdd\xdd\x18\x3f\xe3\x3e\x27\x3e\x07\xff\xff\xff"
"\x4f\xde\x7e\x3f\x57\x45\x34\x3d\xc3\xdb\xa9\xbd\x07\xff\xff\xff"
"\xb5\x28\x67\x3e\xe7\xf9\x2a\x3f\x31\x8f\x35\xbf\x07\xff\xff\xff"
"\x06\x90\x70\xbc\x74\xa2\x73\xbe\x1e\x9f\x78\xbf\x07\xff\xff\xff"
"\x98\x06\x15\xbf\xb9\xcd\x33\xbf\x2a\xbf\xd1\xbe\x07\xff\xff\xff"
"\xc2\xf1\xbb\xbe\x47\x47\x1d\x3f\xc2\xcc\x32\x3f\x07\xff\xff\xff"
"\x59\xc8\x2e\x3f\x55\x20\xcb\xbe\xba\x12\x1d\x3f\x07\xff\xff\xff"
"\x1a\x84\x3e\x3f\xd6\xe9\x27\x3f\xe4\x4a\x01\xbe\x07\xff\xff\xff"
"\x4b\x35\x82\xbe\xa5\x06\x86\x3e\xa6\x57\x6e\xbf\x07\xff\xff\xff"
"\xed\x24\xec\xbe\x8b\xa5\xa4\xbd\x3b\x36\x62\xbf\x07\xff\xff\xff"
"\x33\x3b\xec\xbe\x82\xa7\x5e\xbf\x13\x56\x33\x3e\x07\xff\xff\xff"
"\x0d\xaa\xbe\xbe\xe2\x5d\x35\xbe\xb3\x38\x69\x3f\x07\xff\xff\xff"
"\xb7\x89\x99\xbe\xa4\xf5\x0c\x3f\xea\x6e\x47\x3f\x07\xff\xff\xff"
"\x76\xcb\xd1\x3d\x99\x90\x7c\xbf\x4d\x38\x02\x3e\x07\xff\xff\xff"
"\x61\x90\xf2\x3e\x71\xb3\x59\x3f\x8c\x60\x6a\xbe\x07\xff\xff\xff"
"\x33\xdb\x18\xbe\xf0\x8e\x9c\x3e\x8f\xb9\x70\xbf\x07\xff\xff\xff"
"\x8b\xa5\x3e\xbf\x8e\xec\x83\x3d\x82\x0d\x2a\xbf\x07\xff\xff\xff"
"\xf1\x7d\x40\xbe\xf6\xf2\xf6\xbe\xeb\x07\x5b\x3f\x07\xff\xff\xff"
"\x9f\x9b\x71\x3f\x02\x59\x18\xbe\x84\x23\x97\x3e\x07\xff\xff\xff"
"\xb9\x63\x94\x3e\xc8\xc9\x2d\xbf\x56\xb6\x2c\xbf\x07\xff\xff\xff"
"\x68\xe5\x9e\xba\xe9\x16\xa0\xbe\x34\x2a\x73\x3f\x07\xff\xff\xff"
"\xcd\xbc\x5a\x3f\x9e\x3b\xf3\xbe\x64\x60\x57\x3e\x07\xff\xff\xff"
"\x23\xe6\xc6\x3e\x8c\x2a\x99\x3e\xe8\x1d\x5f\xbf\x07\xff\xff\xff"
"\x4c\xa2\xd8\xbe\x53\xa4\xef\x3e\x95\x9c\x46\xbf\x07\xff\xff\xff"
"\x34\x07\xf0\xbe\xac\xd7\x49\x3f\xd3\xe3\xcb\x3e\x07\xff\xff\xff"
"\x86\x52\x2f\xbf\x21\xe9\xf5\xbe\x22\x49\x0c\x3f\x07\xff\xff\xff"
"\x05\xf6\x3a\xbf\x35\x27\x98\xbd\xb5\xd6\x2d\x3f\x07\xff\xff\xff"
"\xd1\x47\xb3\x3e\x4f\x79\xd9\x3e\x19\xb9\x55\x3f\x07\xff\xff\xff"
"\x89\x14\x0f\x3f\x63\x82\x53\x3f\xf2\xee\x90\x3d\x07\xff\xff\xff"
"\x95\x7e\x79\x3f\x90\x76\x39\x3e\x8e\xf6\x06\x3e\x07\xff\xff\xff"
"\xce\x01\x09\xbd\x05\xc2\x83\x3a\x91\xdb\x7f\xbf\x07\xff\xff\xff"
"\x05\xf4\x17\xbf\x46\xb4\xde\xbe\x77\x57\x2d\xbf\x07\xff\xff\xff"
"\x1e\x60\xfa\xbd\x13\xb0\x40\x3f\xc0\x9d\x25\x3f\x07\xff\xff\xff"
"\x95\x92\x8d\x3e\x92\x90\x4d\x3f\x78\x29\x07\xbf\x07\xff\xff\xff"
"\x56\x7b\xa9\xbd\x9a\x74\x09\xbe\x56\xcc\x7c\xbf\x07\xff\xff\xff"
"\xe4\x59\x40\xbf\x38\x0f\x1c\x3f\x0d\x58\x81\xbe\x07\xff\xff\xff"
"\x37\xe2\x4f\x3f\xa8\x08\x10\xbf\x0b\xb9\x1e\x3e\x07\xff\xff\xff"
"\x91\x96\x7a\x3f\xc9\xb7\x4b\xbe\xd7\x47\x42\x3d\x07\xff\xff\xff"
"\x68\x13\x79\x3e\x66\xc8\x8b\x3e\xf4\x45\x6e\xbf\x07\xff\xff\xff"
"\xf6\x02\xff\x3d\x7e\x46\x78\x3f\x0f\xa9\x56\x3e\x07\xff\xff\xff"
"\xaa\x02\xb7\x3e\x59\x67\xde\xba\x83\x16\x6f\x3f\x07\xff\xff\xff"
"\x65\x8b\x0d\x3e\x0b\x82\x57\xbf\x7b\x91\x05\x3f\x07\xff\xff\xff"
"\x17\x78\xbe\x3e\xce\x07\x67\x3f\x2d\x6b\x5e\x3e\x07\xff\xff\xff"
"\xab\x98\x30\x3f\xcb\xa0\x08\x3f\x34\x76\xfa\x3e\x07\xff\xff\xff"
"\x27\x44\xa5\x3e\xee\x0f\x66\xbf\xcf\x0b\x98\x3e\x07\xff\xff\xff"
"\x20\xee\x7c\x3f\xec\xe9\x15\xbe\x50\xb4\x48\x3d\x07\xff\xff\xff"
"\xe0\xe7\xbc\x3e\x69\xed\x6d\xbf\x6f\x9c\x18\xbc\x07\xff\xff\xff"
"\xf9\x73\x02\x3f\x67\x85\x07\x3f\xd7\xa4\x2d\xbf\x07\xff\xff\xff"
"\x43\xd8\xa8\x3e\x25\x34\x2a\x3f\x95\x94\x2b\xbf\x07\xff\xff\xff"
"\xcf\x22\xb4\x3e\x30\x61\xfe\xbe\x0c\x18\x4b\xbf\x07\xff\xff\xff"
"\xae\x04\x34\xbf\x4e\x40\xeb\xbe\xcb\xe6\x0a\x3f\x07\xff\xff\xff"
"\xe9\x58\x92\xbe\x53\x16\x01\xbf\x54\x9c\x50\x3f\x07\xff\xff\xff"
"\x97\xe6\x52\xbe\xdf\x6e\xf2\xbe\x8b\x3b\x5b\x3f\x07\xff\xff\xff"
"\x59\x39\xc2\x3c\x75\xca\x7f\x3f\xba\xa9\x06\x3d\x07\xff\xff\xff"
"\x66\xfe\x07\x3f\x39\x2e\x16\x3f\x88\x7c\x1c\x3f\x07\xff\xff\xff"
"\xc0\x92\x30\x3f\x6d\x64\x24\x3f\x0d\x45\xab\x3e\x07\xff\xff\xff"
"\x79\x71\x21\x3f\x18\xea\x40\x3f\xb4\x01\x3e\x3e\x07\xff\xff\xff"
"\x15\x08\xe3\xbe\xf8\x9d\x46\xbf\xd6\xcc\xe5\xbe\x07\xff\xff\xff"
"\xca\x88\xa6\xbd\x9e\x33\x72\xbf\x8d\x84\xa0\x3e\x07\xff\xff\xff"
"\x54\x9a\x2a\x3f\xd1\x9d\x30\xbe\xea\xb0\x39\x3f\x07\xff\xff\xff"
"\x01\x7a\xf1\x3e\x46\xb8\x5d\xbf\x16\xa9\x29\xbe\x08\xff\xff\xff"
"\x89\x5a\x6c\xbd\x83\x88\x1a\x3f\x3c\x90\x4b\x3f\x08\xff\xff\xff"
"\xdd\x18\x5a\x3f\xcf\x6d\xda\x3d\x94\x3d\x03\x3f\x08\xff\xff\xff"
"\x96\x72\x47\x3e\x93\x26\x2d\xbe\xc6\x56\x77\xbf\x08\xff\xff\xff"
"\x54\xe9\x11\xbf\x57\x3c\x65\xbe\xf4\x63\x4a\xbf\x08\xff\xff\xff"
"\x7b\xbf\xeb\x3e\x44\xa6\x75\xbe\x15\xcb\x5a\x3f\x08\xff\xff\xff"
"\x57\xa1\x29\x3f\xb9\x1c\x3d\xbf\xcd\xc6\xfc\xbd\x08\xff\xff\xff"
"\xab\x53\x38\xbf\x99\x67\x8f\x3e\x90\x89\x22\xbf\x08\xff\xff\xff"
"\x5c\x46\x79\xbf\x62\xa0\x68\x3e\x55\x64\x7c\xbc\x08\xff\xff\xff"
"\x4a\x0b\x23\x3f\x66\x3f\x2c\x3f\x41\xb4\xc0\x3e\x08\xff\xff\xff"
"\xac\xc0\x04\x3f\x84\xf6\x4f\xbf\x7d\x98\x88\xbe\x08\xff\xff\xff"
"\xbc\xc2\xcc\x3e\x0b\x4a\x39\x3f\xeb\xf3\x0f\xbf\x08\xff\xff\xff"
"\xeb\x6a\xa2\xbe\xcb\x73\xd0\xbe\xbe\x44\x5b\xbf\x08\xff\xff\xff"
"\x64\x05\x5c\xbe\x72\x3f\x77\xbf\x07\x92\x14\xbe\x08\xff\xff\xff"
"\x3d\x3c\xea\xbe\xd7\xae\x5d\x3f\x88\xf1\x4e\x3e\x08\xff\xff\xff"
"\x73\xb7\xb1\xbe\x96\x37\xe9\xbe\x32\xdd\x51\x3f\x08\xff\xff\xff"
"\x1c\x46\x6c\x3f\xa8\xdf\xbd\x3e\x82\x47\xd3\x3d\x08\xff\xff\xff"
"\x85\xf9\x39\x3e\x94\x26\x26\xbf\x8a\x20\x3d\xbf\x08\xff\xff\xff"
"\x2b\xe1\x18\xbe\xca\xdb\x3e\xbf\xb3\x47\x26\xbf\x08\xff\xff\xff"
"\x81\xb8\xad\xbe\x99\xf6\x19\xbf\x13\x2b\x39\xbf\x08\xff\xff\xff"
"\xa6\x85\x07\xbf\x51\x46\x08\xbf\x23\x1d\x29\xbf\x08\xff\xff\xff"
"\x4d\xd2\x21\xbf\xae\xee\x3e\xbf\xb2\x36\x57\x3e\x08\xff\xff\xff"
"\x9d\x83\xa2\xbe\x72\x68\x7a\xbe\x4f\x8d\x6a\x3f\x08\xff\xff\xff"
"\x8a\xbc\x12\x3e\x7a\x44\x6f\xbf\x76\xa2\xa6\x3e\x08\xff\xff\xff"
"\x8d\xbf\x90\xbd\xf3\x31\x0d\x3f\x38\xc6\x54\xbf\x08\xff\xff\xff"
"\xcd\x8b\x25\xbf\x77\xeb\x19\xbf\x4b\x54\xf0\xbe\x08\xff\xff\xff"
"\x05\xbd\x13\xbf\x65\x11\x25\x3f\xf7\x4d\x00\xbf\x08\xff\xff\xff"
"\xca\xc7\xdc\xbe\x44\x5c\x63\xbf\xd0\xdc\x22\xbe\x08\xff\xff\xff"
"\x10\x12\x2e\x3e\xca\x52\x15\x3e\x78\x7f\x79\x3f\x08\xff\xff\xff"
"\x70\x80\xb5\x3e\xd3\x9a\x18\xbf\x9b\x6c\x38\x3f\x08\xff\xff\xff"
"\xee\x84\x2c\x3f\x7a\xe9\x32\xbf\x26\x65\x75\x3e\x08\xff\xff\xff"
"\x79\x1c\x6c\x3f\xdf\xf7\xc4\xbe\x76\x35\x16\x3d\x08\xff\xff\xff"
"\x8a\x18\x8c\x3e\x15\x8b\x6a\x3f\xff\xeb\x95\xbe\x08\xff\xff\xff"
"\xa2\x23\x6d\xbe\x3c\xe3\x4b\xbe\x1a\xc5\x73\xbf\x08\xff\xff\xff"
"\x59\x05\x64\xbe\x82\x6a\x72\xbf\xd2\x5e\x6d\xbe\x08\xff\xff\xff"
"\xb5\xad\xfd\x3e\xcc\xdd\x43\x3f\x57\x91\xd2\x3e\x08\xff\xff\xff"
"\x1d\x4d\xf9\x3e\xf1\x15\x5e\xbf\x6b\x13\xd0\x3d\x08\xff\xff\xff"
"\xb9\xd8\x33\x3f\xcd\x9c\xe7\xbe\x83\xa4\x0c\xbf\x08\xff\xff\xff"
"\x9f\x96\xb4\x3e\x19\xc4\x54\xbf\x22\x21\xdc\xbe\x08\xff\xff\xff"
"\xc3\x76\xbd\xbe\x78\x5a\x6b\xbf\xf2\xe3\x08\xbe\x08\xff\xff\xff"
"\xc0\xe4\x9e\xbe\x86\x3b\x73\xbf\xc6\x53\xfd\x3c\x08\xff\xff\xff"
"\x31\x2a\x09\xbf\x44\xbf\x05\xbf\x05\xce\x29\x3f\x08\xff\xff\xff"
"\x85\xc2\x1f\x3f\xaa\xad\xee\x3e\x98\x88\x20\xbf\x08\xff\xff\xff"
"\x14\x2c\x38\xbf\xf1\x42\xf9\xbe\x44\xa5\xfd\xbe\x08\xff\xff\xff"
"\x03\xe3\xe8\xbe\x8d\xe3\x5d\x3f\x39\x7c\x51\x3e\x08\xff\xff\xff"
"\x62\xbd\x0e\xbe\x38\x2d\x68\x3f\xcb\x8a\xcb\x3e\x08\xff\xff\xff"
"\xd2\xb2\x98\xbe\xe9\xa6\x65\xbe\x83\x82\x6d\x3f\x08\xff\xff\xff"
"\xc0\xa1\x88\x3d\x8c\x65\x6a\xbf\xb6\x01\xcb\x3e\x08\xff\xff\xff"
"\x91\xe2\xa6\x3e\x1e\xdd\x25\xbf\x19\x3f\x30\x3f\x08\xff\xff\xff"
"\x2e\xea\x26\x3f\x6b\x00\x09\x3f\x19\x80\x09\x3f\x08\xff\xff\xff"
"\x09\x4e\x73\x3f\xfd\xbb\x86\xbe\x39\xb6\x29\x3e\x08\xff\xff\xff"
"\x18\xf1\x7b\x3f\x5e\x88\x6b\x3d\x16\xc8\x2b\xbe\x08\xff\xff\xff"
"\x23\x76\x1f\x3e\x8e\x88\x2a\xbe\x27\x42\x79\xbf\x08\xff\xff\xff"
"\xa6\xf6\x28\xbf\xdd\x4b\xe7\x3e\x31\xaa\x19\xbf\x08\xff\xff\xff"
"\xd5\x6a\x68\xbf\x44\x59\xb6\xbe\x60\x75\x62\x3e\x08\xff\xff\xff"
"\xca\xc8\x53\xbe\x5f\xb8\x30\xbf\xa7\x7d\x31\x3f\x08\xff\xff\xff"
"\x10\x17\xd2\x3e\xe5\xfe\xa9\x3e\x0d\x6f\x59\x3f\x08\xff\xff\xff"
"\xd6\x1b\xd4\x3e\x4c\xd2\xc5\xbc\xec\xea\x68\x3f\x08\xff\xff\xff"
"\x4a\xcc\x27\x3f\xe3\xea\x36\x3f\x2e\x7b\x7a\x3e\x08\xff\xff\xff"
"\xc3\x68\x7a\x3f\x77\x18\xd0\xbc\xb4\x3a\x53\x3e\x08\xff\xff\xff"
"\x52\x81\xa3\x3d\x04\xa2\x4f\x3f\x2b\x59\x14\xbf\x08\xff\xff\xff"
"\x00\x5b\x7b\xbf\xf5\xd3\x2c\x3e\x81\xf4\xb0\x3d\x08\xff\xff\xff"
"\x14\x51\x09\xbf\x2a\x33\x4a\xbf\xd8\x41\x98\x3e\x08\xff\xff\xff"
"\xa6\x9c\x42\x3e\xa0\x9b\x0c\x3e\x74\xdd\x78\x3f\x08\xff\xff\xff"
"\xac\x21\x3c\x3f\x58\xbc\x6b\x3e\xd2\x4e\x23\x3f\x08\xff\xff\xff"
"\xfa\xe8\xa0\x3e\xf5\x3a\xfa\x3e\x71\x59\x50\xbf\x08\xff\xff\xff"
"\x82\x02\x7b\x3e\xdd\xa4\x28\x3f\xa4\x17\x36\xbf\x08\xff\xff\xff"
"\xf8\x25\x96\x3e\x76\x44\x6c\xbe\xdc\x82\x6d\xbf\x08\xff\xff\xff"
"\x59\x7d\xee\xbe\x9d\xc9\x5c\xbf\x63\xc6\x4a\x3e\x08\xff\xff\xff"
"\x7d\x31\xa2\x3e\xa3\xd8\x3e\x3f\x36\x20\x16\x3f\x08\xff\xff\xff"
"\xec\x69\xc7\x3e\xee\x16\x2f\x3f\x45\xeb\x1d\xbf\x08\xff\xff\xff"
"\x7f\x12\x7d\xbf\xd3\x1e\x19\x3e\x74\x21\x9f\xbc\x08\xff\xff\xff"
"\xff\xc9\x16\xbf\xe0\xc7\x06\xbf\xe2\xf2\x1c\x3f\x08\xff\xff\xff"
"\x7a\xc9\xc9\xbe\xfb\xe2\x58\x3f\xf2\x65\xb6\x3e\x08\xff\xff\xff"
"\xda\x31\xec\xbe\x42\x1f\x2f\xbf\x7e\xa4\x10\x3f\x08\xff\xff\xff"
"\x70\xff\x0b\xbf\x26\x0a\xf7\xbe\xec\x27\x2f\x3f\x08\xff\xff\xff"
"\x2c\x44\xd6\xbe\x08\x81\x20\xbf\xb4\x37\x28\x3f\x08\xff\xff\xff"
"\x3b\x04\x5e\x3f\x6f\x6e\xab\x3e\xad\xa6\xbc\xbe\x08\xff\xff\xff"
"\x94\x18\xf1\xbe\x95\xcf\xc9\x3e\xb3\x0b\x4a\xbf\x08\xff\xff\xff"
"\x08\x35\xc3\xbd\xf2\xf2\x3a\xbf\xd8\x2d\x2d\x3f\x08\xff\xff\xff"
"\xb7\x17\x0d\x3f\xcc\xf1\x34\x3f\x38\x0d\xe3\xbe\x08\xff\xff\xff"
"\x09\xa2\x26\x3f\xb3\x69\xd3\xbd\x00\x8a\x40\xbf\x08\xff\xff\xff"
"\xbe\x00\xf4\x3d\x01\x50\x01\x3e\xff\x1c\x7c\xbf\x08\xff\xff\xff"
"\x4a\x9d\xc8\xbe\x7f\x4f\x91\xbe\x8d\x0b\x60\xbf\x08\xff\xff\xff"
"\x0a\xd5\x4e\xbe\x82\x3e\x7a\xbf\x18\x41\x78\xbd\x08\xff\xff\xff"
"\x8b\x45\x19\xbf\x61\x2a\x32\xbf\x78\xfd\xca\xbe\x08\xff\xff\xff"
"\xdb\xa4\x41\xbf\xb6\x54\xbb\x3e\x54\xcc\x0a\x3f\x08\xff\xff\xff"
"\xa6\xf6\x9c\xbe\xd2\x8f\x4a\xbf\x32\x72\x07\x3f\x08\xff\xff\xff"
"\xb4\x01\x47\x3f\xc1\x8e\x8c\xbe\xe3\xe3\x10\x3f\x08\xff\xff\xff"
"\xeb\x7b\xfd\xbe\xe2\x68\x5b\xbf\xa6\x00\x12\xbe\x08\xff\xff\xff"
"\x57\x54\x75\xbf\x9e\x9e\x8e\xbe\x5e\x03\x82\xbd\x08\xff\xff\xff"
"\x03\x30\x77\xbf\x75\xae\x83\xbe\xef\x37\x1f\x3d\x08\xff\xff\xff"
"\x58\xac\x39\xbf\x24\x48\x17\x3a\x24\x3e\x30\x3f\x08\xff\xff\xff"
"\xd3\x6d\x54\x3f\xa6\x62\xb2\x3e\x52\x33\xdf\x3e\x08\xff\xff\xff"
"\xa4\x85\x02\x3f\x89\xc6\x59\x3f\x55\x1a\x03\x3e\x08\xff\xff\xff"
"\x97\x42\x99\xbd\x57\x2d\x15\xbe\x66\x8b\x7c\xbf\x09\xb2\xcc\xff"
"\xc1\xdf\x79\x3f\x5d\x93\x26\xbe\xa5\xc0\x13\x3e\x09\xff\xb2\x7f"
"\x7c\x4f\xc3\x3e\xfa\xbe\x42\x3f\x42\x70\x06\xbf\x09\xff\xb2\x7f"
"\xe7\x72\x13\x3f\xce\x50\xbd\x3e\x49\xa4\x3a\x3f\x09\xff\xb2\x7f"
"\x19\x7b\x9e\xbe\x48\x74\x72\x3f\xb9\x0e\xae\xbd\x09\xff\xb2\x7f"
"\x1e\xde\x45\x3f\x7e\x5d\x8d\xbe\x76\x3f\x12\xbf\x09\xff\xcc\xb2"
"\xe3\x6a\xe3\x3e\xf3\xef\x8c\xbe\xa3\x44\x5a\xbf\x09\xff\xcc\xb2"
"\xa7\xec\xfb\xbe\xe8\x28\x30\xbf\x36\x85\x08\xbf\x09\xff\xcc\xb2"
"\x6e\xa5\xc0\x3d\x45\x54\x12\x3f\x2a\xac\x50\xbf\x09\xff\xcc\xb2"
"\x0e\x21\x18\xbf\xc2\x14\x3e\xbf\x00\x45\x9e\xbe\x09\xff\xcc\xb2"
"\x48\xca\xe7\x3d\x3a\x94\x62\x3f\x22\x2a\xe7\x3e\x09\xff\xcc\xb2"
"\x12\x4f\x83\x3e\xd4\xec\x44\x3e\xd9\x7d\x72\xbf\x09\xff\xcc\xb2"
"\xb7\xa0\xe3\xbd\xd6\xa3\x4b\xbf\x0d\x81\x18\x3f\x09\xff\xcc\xb2"
"\xf4\x67\x1c\x3e\x34\x06\x78\xbf\x56\xa9\x47\x3e\x09\xff\xcc\xb2"
"\x86\x1f\xaa\xbe\x5c\xb9\x6c\xbf\x17\x47\x3e\xbe\x09\xff\xcc\xb2"
"\xa5\x7f\x45\xbe\x93\xfb\x75\xbf\xd0\x9e\x4b\xbe\x09\xff\xcc\xb2"
"\x17\x56\x0d\xbf\xe5\x5d\x55\xbf\x6d\x4e\xbd\x3c\x09\xff\xcc\xb2"
"\x1d\xb2\xff\x3e\x82\x2f\x41\x3f\x03\xe8\xd9\x3e\x09\xff\xcc\xb2"
"\xb6\xbb\xd6\x3e\xa3\xf5\x1e\x3f\xbd\x87\x29\xbf\x09\xff\xcc\xb2"
"\x77\xef\x70\xbf\x2b\xf5\x89\xbe\x30\xea\x50\xbe\x09\xff\xcc\xb2"
"\x22\x3c\x4b\xbf\x11\xd2\x15\x3f\x0f\x03\x29\xbe\x09\xff\xcc\xb2"
"\x9a\x0d\x27\xbe\xe2\x5b\x74\xbf\x04\x88\x7f\x3e\x09\xff\xcc\xb2"
"\xea\x09\x70\x3e\x23\x91\x29\x3e\x1d\x3b\x75\xbf\x09\xff\xcc\xb2"
"\x40\x16\x87\x3e\x0f\xcc\xe6\x3e\x7a\x4e\x5a\x3f\x09\xff\xcc\xb2"
"\x7c\xf5\x09\x3e\x4f\x8b\x72\xbf\x39\x96\x94\xbe\x09\xff\xcc\xb2"
"\x46\x78\x9b\x3e\xc1\x30\x6f\xbd\x6d\x74\x73\x3f\x09\xff\xcc\xb2"
"\x7c\xa0\x3d\x3e\xfa\x98\x40\xbf\xb6\xd9\x21\xbf\x09\xff\xcc\xb2"
"\x9e\xd6\x15\x3e\xe7\x95\x2d\x3f\x34\x65\x38\x3f\x09\xff\xcc\xb2"
"\x09\x72\x30\x3e\xfe\xd7\xb4\xbe\x46\x67\x6b\xbf\x09\xff\xcc\xb2"
"\x40\x32\x72\xbd\xfa\xf0\x28\xbf\xf8\xbe\x3f\xbf\x09\xff\xcc\xb2"
"\x12\x2a\xc0\xbe\xdb\x01\xf5\xbe\xe8\x37\x4b\xbf\x09\xff\xcc\xb2"
"\x8f\x3b\x26\x3f\xb0\xfa\x41\xbf\xd1\xa8\x84\xbd\x09\xff\xcc\xb2"
"\xd5\x7f\x4f\xbf\x66\x17\xe0\x3e\x52\x41\xc7\xbe\x09\xff\xcc\xb2"
"\x03\x0b\x14\x3f\x69\xe6\x86\xbe\x5e\xa9\x45\xbf\x09\xff\xcc\xb2"
"\x1e\x44\x09\x3f\xf7\x59\x1d\x3f\xd8\x1a\x14\x3f\x09\xff\xcc\xb2"
"\x53\x80\x2b\x3f\xe1\xda\x3b\xbf\x2b\x06\xe7\x3d\x09\xff\xcc\xb2"
"\xf4\xf4\x66\xbf\x61\xf2\xc1\xbc\xdf\x86\xdc\xbe\x09\xff\xcc\xb2"
"\x48\xa7\x4a\x3f\xe9\x02\xbd\xbe\xc6\x49\xf9\x3e\x09\xff\xff\xcc"
"\x13\x16\xc5\x3e\x2c\x24\x99\xbe\xdc\x85\x5f\x3f\x09\xff\xff\xcc"
"\x82\xcc\xa5\x3e\x19\xbc\x56\x3f\x62\x15\xe0\x3e\x09\xff\xff\xcc"
"\xf8\xb3\xdd\xbe\xc0\x3e\xcf\x3e\x4a\x2e\x4e\xbf\x09\xff\xff\xcc"
"\x67\xcf\x63\xbf\x2d\x5c\xb4\x3e\x32\x67\x94\xbe\x09\xff\xff\xcc"
"\x05\xec\x0a\xbe\x1f\x03\xe4\xbe\x75\x91\x62\xbf\x09\xff\xff\xcc"
"\x63\x2a\x19\x3e\xc0\xe6\x19\xbf\xc2\xf5\x48\xbf\x09\xff\xff\xcc"
"\xc1\x3a\x38\x3f\x5a\x23\x2f\x3f\xf2\xf1\xf2\xbd\x09\xff\xff\xcc"
"\xb3\xba\x79\x3e\x24\x4f\x74\x3f\x97\xb8\x30\x3e\x09\xff\xff\xcc"
"\x9a\x4d\x77\xbe\xe8\x2e\x1f\xbf\xee\xb8\x3e\xbf\x09\xff\xff\xcc"
"\x70\x08\x2c\xbe\x5d\x1e\x57\xbf\x09\xf3\x03\x3f\x09\xff\xff\xcc"
"\xf4\x8e\x2e\xbf\x4a\xc4\x0e\xbf\xe3\x58\xf2\x3e\x09\xff\xff\xcc"
"\x8f\xd0\xae\xbd\x75\x13\x74\xbf\x26\x22\x94\x3e\x09\xff\xff\xcc"
"\xb8\xa2\x0d\xbf\x42\xc1\x54\xbf\x13\x6b\x68\x3d\x09\xff\xff\xcc"
"\xdd\x9a\xe3\xbe\x30\xfd\x12\xbe\xf4\x59\x62\x3f\x09\xff\xff\xcc"
"\x16\x5a\x0c\x3f\x22\xef\x55\x3f\x32\xc2\x05\x3d\x09\xff\xff\xe5"
"\xc6\x30\x83\xbd\x26\xa3\x6d\xbf\xf3\x93\xbb\xbe\x09\xff\xff\xe5"
"\x2a\xd8\x5f\x3f\xdc\x66\xd5\x3e\x04\x63\x7e\x3e\x09\xff\xff\xe5"
"\x56\x4c\xaa\x3e\x03\x9e\x16\x3f\x37\xae\x3c\xbf\x09\xff\xff\xe5"
"\xbe\x33\xc6\xbe\x22\xf0\x8e\x3e\x04\xf6\x60\xbf\x09\xff\xff\xe5"
"\x64\x80\xde\xbe\xa0\x50\x5a\x3f\x87\x4c\x94\xbe\x09\xff\xff\xe5"
"\xbf\x70\xde\xbe\xb8\x30\x96\x3e\xed\x01\x5a\x3f\x09\xff\xff\xe5"
"\xbf\x70\xde\xbe\xb8\x30\x96\x3e\xed\x01\x5a\x3f\x09\xff\xff\xe5"
"\x4d\x14\x0b\xbf\x02\xc0\x56\xbf\xb9\x3c\x0b\xbd\x09\xff\xff\xe5"
"\x3b\x5a\x0e\xbf\x18\xb6\x3e\xbf\xde\xb1\xbc\xbe\x09\xff\xff\xe5"
"\xf6\xe2\x2f\xbf\x99\x81\x39\xbf\x94\x1d\x5b\xbd\x09\xff\xff\xe5"
"\xed\x4e\xad\xbe\x9e\x7f\xc1\x3e\x47\x9b\x5c\x3f\x09\xff\xff\xe5"
"\x79\xd1\xe4\x3e\x84\x72\x14\xbf\xd8\x62\x2e\xbf\x09\xff\xff\xe5"
"\x04\xc8\x23\xbf\x51\xe7\x30\x3c\xe1\xbb\x44\x3f\x09\xff\xff\xe5"
"\x7d\x4c\x1c\x3f\x6b\x4a\x94\x3d\x3b\xe6\x49\x3f\x09\xff\xff\xe5"
"\x24\xf8\xf5\x3e\x95\xc6\x65\x3d\x7c\x10\x60\xbf\x09\xe5\xe5\xff"
"\xa4\xf3\x3d\x3f\x8a\xd8\x9b\x3e\xd7\xe9\x18\x3f\x09\xe5\xe5\xff"
"\x45\x7e\xaf\x3d\xbb\xe2\x48\x3f\x03\x2a\x1d\x3f\x09\xe5\xe5\xff"
"\x5f\x5b\xbf\x3e\xa5\x50\xde\x3e\xd1\xd2\x51\xbf\x09\xe5\xe5\xff"
"\xbc\x2c\x1b\xbf\x67\x48\x17\xbf\xf3\x45\x08\x3f\x09\xe5\xe5\xff"
"\x6f\xa6\x47\x3e\x0a\x56\x0f\x3f\xeb\x27\x4e\x3f\x09\xe5\xe5\xff"
"\x81\x67\xdd\xbe\xed\xed\x60\xbf\x76\x65\x4f\xbe\x09\xe5\xe5\xff"
"\xb5\x16\xb6\xbe\x2b\xd7\x0a\x3f\x79\xdc\x42\x3f\x09\xe5\xe5\xff"
"\x6c\x81\xe2\xbe\x3b\xa2\x02\xbf\xa8\xcc\x3c\xbf\x09\xe5\xe5\xff"
"\x9c\xaf\x74\xbf\x6b\xd7\x3a\x3e\xfe\x0d\x6c\xbe\x09\xe5\xe5\xff"
"\xac\xa5\x44\x3e\x5b\x1f\x8b\xbe\x00\x6a\x71\x3f\x09\xe5\xe5\xff"
"\xad\x6a\xc7\x3e\x06\xb9\x9c\x3e\xf3\x61\x5e\xbf\x09\xe5\xe5\xff"
"\x62\xdd\xab\xbe\xf6\x2c\x6c\xbf\xd0\xdb\x42\x3e\x09\xe5\xe5\xff"
"\x17\x33\x92\xbe\x84\x56\x75\x3f\xb1\xb2\xbf\x3b\x09\xe5\xe5\xff"
"\x0c\xf6\x14\xbf\x3d\xd1\x20\xbf\x9a\x3a\x04\x3f\x09\xe5\xe5\xff"
"\x18\xf4\xc6\xbe\xc6\x2a\x2c\xbf\x0e\x3e\x21\x3f\x09\xe5\xe5\xff"
"\xdc\x76\x58\xbf\x4f\xb8\xc0\x3c\xec\x89\x08\xbf\x09\xe5\xe5\xff"
"\x2f\x55\xad\xbe\x22\x76\xc5\xbe\xab\xb8\x5b\x3f\x09\xe5\xe5\xff"
"\x54\x15\x1e\xbf\xd9\x53\x48\xbf\x5f\xed\xa2\x3d\x09\xe5\xe5\xff"
"\x70\x72\x1b\xbe\xe3\xe1\x4b\x3f\x9b\xdc\x15\x3f\x09\xe5\xe5\xff"
"\x8d\xe3\x25\xbf\x7e\x78\x37\xbf\x28\xfe\x83\x3e\x09\xe5\xe5\xff"
"\xfe\x36\x4e\x3e\x3b\xa3\x83\x3e\xde\xf6\x71\x3f\x09\xff\xff\xff"
"\xf8\x0c\x6b\xbe\x8a\x73\xdd\x3e\x33\x36\x5f\x3f\x09\xff\xff\xff"
"\x12\x93\xea\x3d\x98\xde\x14\x3e\x14\x94\x7b\x3f\x09\xff\xff\xff"
"\x8d\x3f\x43\x3f\xad\xfd\x09\xbe\x33\xf0\x21\x3f\x09\xff\xff\xff"
"\x75\x79\x28\x3f\x2c\x16\x40\x3f\x85\xaa\x7f\x3d\x09\xff\xff\xff"
"\x41\xe6\xb9\x3e\xd6\xe5\x51\x3f\x75\xa0\xe2\x3e\x09\xff\xff\xff"
"\xf7\xd9\x28\xbf\x9a\x6e\x15\x3f\xc2\x72\xf2\xbe\x09\xff\xff\xff"
"\x63\x8d\x60\xbe\x8d\xaa\x15\x3e\x9c\xf3\x76\x3f\x09\xff\xff\xff"
"\x4c\x88\x76\x3f\xd2\x0c\x27\xbd\xfd\x5b\x88\x3e\x09\xff\xff\xff"
"\x76\xe2\xd6\xbd\xb2\xb9\x55\xbf\xf5\x55\x0a\xbf\x09\xff\xff\xff"
"\xd0\x6e\x3d\xbf\x6b\x09\x28\xbf\xe8\x76\x16\xbe\x09\xff\xff\xff"
"\x20\xfd\x28\xbf\x67\x8c\x31\xbf\xd3\xb8\x93\x3e\x09\xff\xff\xff"
"\x10\x95\x3e\x3f\x06\x8a\x0d\x3f\xb0\xa2\xbf\xbe\x09\xff\xff\xff"
"\x2d\xbe\x77\xbf\x04\xd8\x70\x3d\x6f\xd7\x7a\x3e\x09\xff\xff\xff"
"\x41\x1c\x38\xbf\x28\x82\xdb\xbd\xf3\xbe\x2f\xbf\x09\xff\xff\xff"
"\x67\x20\xae\x3e\x5b\xf4\x1d\xbc\x91\xba\x70\xbf\x09\xff\xff\xff"
"\xfb\x6b\x12\xbf\x7b\x7e\x3c\xbf\xf7\x1d\xb9\xbe\x09\xff\xff\xff"
"\x6e\xc8\x47\xbd\xd0\xe0\x91\x3d\x97\x0b\x7f\xbf\x0a\xb2\xcc\xff"
"\x99\x48\xe9\x3e\x77\x4c\xf2\xbe\x5d\x03\x41\xbf\x0a\xff\xb2\x7f"
"\x79\xfa\xb8\x3e\xfd\x48\x92\xbe\x2d\x3a\x63\x3f\x0a\xff\xcc\xb2"
"\xf4\x5c\x2c\xbf\xf4\x93\xeb\x3d\xc9\xf9\x3a\xbf\x0a\xff\xcc\xb2"
"\x90\xcb\xe1\x3e\xa7\x90\x65\x3f\x22\x29\x18\xbd\x0a\xff\xcc\xb2"
"\xcc\x94\xce\x3e\xe5\xaf\x8f\xbe\xd7\xf2\x5e\xbf\x0a\xff\xcc\xb2"
"\x3d\x00\x73\x3f\x4d\xc3\x62\xbe\x53\xd2\x64\xbe\x0a\xff\xcc\xb2"
"\xb5\x5f\xdc\xbe\xb9\x3b\xfe\xbe\xa6\xf7\x40\x3f\x0a\xff\xcc\xb2"
"\x3b\xe8\x14\x3f\x3f\x92\x2e\x3f\x35\x0c\xe3\xbe\x0a\xff\xcc\xb2"
"\x1a\x3a\x0f\xbf\x15\x61\x18\x3f\x98\xa8\x13\x3f\x0a\xff\xcc\xb2"
"\xe1\x6a\xd2\xbe\x6b\x61\xbc\xbe\xfb\x87\x55\x3f\x0a\xff\xcc\xb2"
"\x4c\x4f\x04\xbf\xb8\x32\x54\xbf\xed\x2c\x5b\x3e\x0a\xff\xcc\xb2"
"\x76\x6d\x58\x3f\x8a\x31\x6e\xbe\xb6\x29\xf6\xbe\x0a\xff\xcc\xb2"
"\xaf\x6c\xfa\xbe\x1d\x2e\x28\xbf\x6a\xe1\x12\xbf\x0a\xff\xcc\xb2"
"\x4b\x51\x70\xbf\xf1\x26\x4e\xbe\x1f\x35\x8f\xbe\x0a\xff\xcc\xb2"
"\x9a\xf3\xa8\xbe\x93\x51\x6a\xbf\x8d\x79\x6c\x3e\x0a\xff\xcc\xb2"
"\xfb\x87\xf3\xbd\xd7\xbb\x16\xbf\x01\xab\x4c\x3f\x0a\xff\xcc\xb2"
"\x04\x85\x31\x3b\x71\x1c\x5b\xbf\xfb\x62\x04\x3f\x0a\xff\xcc\xb2"
"\x72\x5e\x8a\x3e\x4e\xaf\x22\x3e\xa2\x18\x73\xbf\x0a\xff\xcc\xb2"
"\x89\x5f\xc2\x3e\x82\x3f\x55\xbf\x2b\x12\xce\x3e\x0a\xff\xcc\xb2"
"\xdf\xe3\x69\x3f\xfd\x99\xca\xbe\x12\xe3\xbe\x3d\x0a\xff\xcc\xb2"
"\xbd\xec\x7f\x3f\x2e\x82\x5c\x3c\xbf\x17\xa4\x3c\x0a\xff\xcc\xb2"
"\x1f\x2b\x22\xbf\x35\x11\x46\x3f\x43\x15\x29\xbc\x0a\xff\xcc\xb2"
"\x53\x6e\x6c\xbd\x52\x2c\x40\x3f\x5e\x7c\x28\xbf\x0a\xff\xcc\xb2"
"\x34\x5b\x20\x3f\xcb\x53\x34\x3f\x24\xee\xaa\x3e\x0a\xff\xcc\xb2"
"\xbe\x8c\x6e\x3e\x77\xcd\x61\x3e\x0e\x79\x72\xbf\x0a\xff\xff\xcc"
"\x77\xcb\xd3\xbd\x07\xce\x35\x3f\x18\x47\x32\x3f\x0a\xff\xff\xcc"
"\x29\x26\x5f\xbf\x61\x88\xf6\xbe\xc5\xe4\xba\x3d\x0a\xff\xff\xcc"
"\x45\x3d\x7a\xbf\xcd\xb3\x57\xbe\x0e\xf7\x34\x3c\x0a\xff\xff\xcc"
"\xdb\xad\x67\x3e\x3e\xbd\x63\xbf\xf8\x23\xcb\x3e\x0a\xff\xff\xcc"
"\x08\xbe\x02\xbf\x6d\xd3\xb6\x3e\x33\x37\x48\xbf\x0a\xff\xff\xcc"
"\x19\xd7\x2a\x3f\x0e\x8c\x31\xbe\xc9\x6a\x39\x3f\x0a\xff\xff\xcc"
"\x98\xeb\xfa\x3e\xf9\x0a\x34\x3d\xcd\xdd\x5e\xbf\x0a\xff\xff\xcc"
"\x8b\x6e\x53\x3f\x63\x3d\x30\x3e\xfc\x71\x09\xbf\x0a\xff\xff\xcc"
"\x80\x49\xec\xbe\x3c\xb9\x61\xbf\xac\x63\xc8\xbd\x0a\xff\xff\xcc"
"\x9d\x25\xe6\x3e\x83\x5d\x57\xbf\xd6\xc8\x99\xbe\x0a\xff\xff\xcc"
"\x52\x70\x21\x3e\xe1\xb8\x11\x3f\x02\x93\x4e\x3f\x0a\xff\xff\xcc"
"\x0a\xe9\x1e\x3f\x48\xe7\x27\x3e\x0d\x45\x44\x3f\x0a\xff\xff\xcc"
"\xe9\x0c\xe1\xbe\xec\xd9\x9d\x3e\x85\xf9\x57\xbf\x0a\xff\xff\xe5"
"\x44\x9e\x58\x3d\xfd\x5f\x2a\xbe\xbf\x11\x7c\x3f\x0a\xff\xff\xe5"
"\x49\x84\xf4\x3d\x98\xa8\x79\x3f\x40\xb5\x3e\x3e\x0a\xff\xff\xe5"
"\x18\x65\xdd\xbd\xdc\x5c\xf1\xbe\xa7\x11\x60\xbf\x0a\xff\xff\xe5"
"\xaa\xae\x33\xbf\x4e\xb9\x26\xbf\xb2\xb2\x93\xbe\x0a\xff\xff\xe5"
"\x54\x40\x7d\xbf\xa4\xb1\xb5\x3d\x8d\xce\xed\x3d\x0a\xff\xff\xe5"
"\x44\x60\x3d\x3f\x4a\x27\x91\x3e\xf9\x38\x1c\x3f\x0a\xff\xff\xe5"
"\xc7\xa8\x70\xbf\x8e\x32\x5a\x3d\xac\x6d\xac\x3e\x0a\xff\xff\xe5"
"\xcc\x59\xb8\xbe\x6c\xcd\x61\x3f\x1d\x9b\x9b\xbe\x0a\xff\xff\xe5"
"\xc7\xa8\x70\xbf\x8e\x32\x5a\x3d\xac\x6d\xac\x3e\x0a\xff\xff\xe5"
"\xaf\x3e\xca\xbe\x8f\x0e\x15\x3f\x25\xea\x35\x3f\x0a\xff\xff\xe5"
"\x59\xc6\x59\x3f\xd4\x1f\x90\xbe\x5b\x4e\xe3\x3e\x0a\xff\xff\xe5"
"\x51\x23\x20\xbe\x69\x69\xac\xbe\xd6\xb3\x6d\xbf\x0a\xff\xff\xe5"
"\x2b\x5a\x32\x3f\x9f\x11\x20\x3f\x67\x0d\xb4\x3e\x0a\xff\xff\xe5"
"\xa0\xe1\x24\x3e\x7c\x45\x27\xbf\x56\x5c\x3d\x3f\x0a\xe5\xe5\xff"
"\x11\xc2\x13\xbf\x9e\x85\x4e\xbf\x52\xc6\x01\xbe\x0a\xe5\xe5\xff"
"\x68\x5c\x07\xbf\xc2\x5b\x24\xbf\x53\x20\x0e\x3f\x0a\xe5\xe5\xff"
"\x18\xe5\xd3\x3e\xcc\xd9\x57\xbf\x99\xba\xaf\xbe\x0a\xe5\xe5\xff"
"\xbf\xaf\x50\x3e\x38\x7f\x25\xbf\x8f\x37\x3c\xbf\x0a\xe5\xe5\xff"
"\x2b\xfe\x04\xbe\xf9\x8c\x06\xbf\xb2\x3c\x57\x3f\x0a\xe5\xe5\xff"
"\x6f\x16\x5c\x3c\xef\x7b\x64\xbe\x0a\x86\x79\xbf\x0a\xe5\xe5\xff"
"\xb2\x88\xfd\xbe\x0c\xfa\x4f\xbf\xf4\xa2\x9d\xbe\x0a\xe5\xe5\xff"
"\x7a\x44\xa7\xbc\xb2\xeb\x50\xbf\xf3\xda\x13\x3f\x0a\xe5\xe5\xff"
"\xbf\x34\x44\x3f\x4e\x6f\x24\x3f\x0e\xa4\x2c\xbb\x0a\xe5\xe5\xff"
"\xde\x57\x11\x3f\xa2\xd1\xb6\x3e\x36\xe2\x3d\xbf\x0a\xe5\xe5\xff"
"\x48\x86\xc1\x3e\x65\x1a\x9a\x3e\x83\x22\x60\xbf\x0a\xe5\xe5\xff"
"\x1d\x2f\x18\x3f\xd7\xec\x41\x3f\x89\x1e\x8a\xbe\x0a\xe5\xe5\xff"
"\x8e\x1b\xcd\xbe\x98\xfa\xc9\xbe\x25\xb5\x53\x3f\x0a\xe5\xe5\xff"
"\x92\x2d\xb6\x3d\x82\xbb\x33\xbf\xc0\xde\x34\x3f\x0a\xe5\xe5\xff"
"\x86\x5e\x2b\xbf\xe2\xea\x11\xbf\x45\xf1\xf3\xbe\x0a\xe5\xe5\xff"
"\x29\xb4\x0c\x3e\xe0\xe3\x45\xbc\xd2\x8d\x7d\xbf\x0a\xe5\xe5\xff"
"\xac\xa5\xc9\x3e\xec\xaa\x9a\x3e\x4c\x3d\x5e\xbf\x0a\xe5\xe5\xff"
"\x77\xdc\xfa\x3d\xdf\xd2\x88\xbd\x3e\x7f\x7d\xbf\x0a\xe5\xe5\xff"
"\xb8\xe2\x61\xbf\xd7\xcb\x2b\x3e\x29\x18\xe1\xbe\x0a\xff\xff\xff"
"\xfd\x96\x64\xbf\x0e\x82\x97\x3e\xcc\xb5\xad\xbe\x0a\xff\xff\xff"
"\x0a\xc8\xc9\xbe\xdd\xa0\x5e\x3d\xb5\xde\x6a\x3f\x0a\xff\xff\xff"
"\xc7\x1a\x73\x3f\xec\xe8\xc4\x3d\xb5\xb5\x98\x3e\x0a\xff\xff\xff"
"\x7e\x35\xef\xbd\x87\xf4\xb2\xbe\xe9\xfb\x6d\xbf\x0a\xff\xff\xff"
"\x80\x1f\xe7\xbe\xdd\x5f\x5d\x3e\xb9\xa0\x5d\xbf\x0a\xff\xff\xff"
"\x3d\x74\x96\xbe\x3e\xc4\xfa\x3e\x3f\x22\x52\xbf\x0a\xff\xff\xff"
"\xde\x94\x06\xbf\xd5\xaf\x36\x3f\xa1\x0f\xed\xbe\x0a\xff\xff\xff"
"\xa3\x8b\xde\xbe\x85\xf9\x63\xbf\xec\x89\x09\x3e\x0a\xff\xff\xff"
"\x7d\xb9\x92\xbe\x35\xaf\x70\x3f\x8c\xb4\x3c\x3e\x0a\xff\xff\xff"
"\x00\xda\xc1\xbe\x77\x89\xbf\x3e\xfa\xb9\x58\x3f\x0a\xff\xff\xff"
"\x97\xc4\xc3\xbe\x02\xd6\x43\x3e\x32\x6d\x67\x3f\x0a\xff\xff\xff"
"\x41\x37\x20\xbf\xfa\x04\x12\x3e\x58\x4d\x44\x3f\x0a\xff\xff\xff"
"\xc4\x62\x46\xbe\xbd\x80\x97\x3e\x43\x74\x6f\xbf\x0b\xff\xb2\x7f"
"\x47\xf2\x07\xbf\x2d\x81\xa5\xbe\x30\x84\x48\x3f\x0b\xff\xb2\x7f"
"\x9e\x9c\x69\xbf\x22\x96\x11\x3e\x2e\x55\xc4\x3e\x0b\xff\xb2\x7f"
"\x14\xd6\xee\x3e\x3c\x1e\x09\x3f\xaf\x34\x34\xbf\x0b\xff\xb2\x7f"
"\x16\x71\x16\x3f\xc5\xe1\x3f\xbf\x40\x03\x9c\xbe\x0b\xff\xb2\x7f"
"\x3c\x4d\x1c\x3f\x19\xdd\xc2\x3e\x0c\xcd\x31\x3f\x0b\xff\xb2\x7f"
"\x1c\x11\xa1\xbc\xa2\xed\xb2\x3e\xad\xce\x6f\x3f\x0b\xff\xb2\x7f"
"\x6f\x21\x55\x3f\xe2\xe5\xe2\x3e\xce\x2e\xaa\xbe\x0b\xff\xcc\xb2"
"\xb8\x7a\x31\xbf\xc9\x39\xb5\xbe\xb8\xb4\x20\x3f\x0b\xff\xcc\xb2"
"\xb8\x7a\x59\xbe\x45\xe9\x68\x3f\x13\x92\xb6\x3e\x0b\xff\xcc\xb2"
"\x2f\x66\xbd\x3e\xe5\xd2\xca\xbe\x0c\x23\x57\x3f\x0b\xff\xcc\xb2"
"\xa8\x98\x1a\x3e\x45\x90\x97\xbd\x45\x5b\x7c\x3f\x0b\xff\xcc\xb2"
"\xa5\x44\x0a\x3f\x49\x9e\x56\x3f\xd1\x36\x97\x3d\x0b\xff\xcc\xb2"
"\x23\x20\x00\xbf\x93\x58\x5d\xbf\x65\xbe\x33\xbd\x0b\xff\xcc\xb2"
"\x74\xd1\xcf\x3e\x88\xea\x63\x3f\xe5\x6b\x53\x3e\x0b\xff\xcc\xb2"
"\xc2\xeb\x5e\x3f\xc3\xfa\x85\xbe\x87\x1d\xd5\xbe\x0b\xff\xcc\xb2"
"\xd9\x6c\xc9\xbe\xf9\x31\x08\xbf\xfb\xf2\x3f\xbf\x0b\xff\xcc\xb2"
"\xfc\xd1\x4f\xbf\x43\x4b\x14\x3f\x87\xda\x96\xbd\x0b\xff\xcc\xb2"
"\x89\x05\x15\x3f\x80\x5f\x48\xbf\xc1\x8c\x61\xbe\x0b\xff\xff\xcc"
"\x51\x05\x10\xbf\xd3\x15\xf0\xbe\x8d\x4f\x2e\xbf\x0b\xff\xff\xcc"
"\x6c\xda\xbf\x3d\xef\xec\x6e\xbf\x0c\x7f\xb1\xbe\x0b\xff\xff\xcc"
"\xd4\xbd\xa2\xbe\x49\x17\x59\x3f\xee\x21\xd9\xbe\x0b\xff\xff\xcc"
"\x86\xbf\x44\xbf\xa0\x7f\xa6\x3d\x91\x75\x22\xbf\x0b\xff\xff\xcc"
"\xa6\xfe\xba\xbe\x39\xf9\x36\xbf\x70\xb1\x18\x3f\x0b\xff\xff\xcc"
"\xf2\x96\xec\x3d\xd1\xb0\x0c\x3f\x48\xd2\x53\x3f\x0b\xff\xff\xcc"
"\x97\x75\x08\x3f\x1c\x2f\x4c\x3f\xd7\x8d\x90\xbe\x0b\xff\xff\xcc"
"\x8a\x4b\xd3\xbe\x80\xd0\x23\x3f\xb0\xf3\x25\x3f\x0b\xff\xff\xcc"
"\x79\x21\x6b\x3f\x18\x91\xc5\x3e\xa4\x4d\xb1\x3d\x0b\xff\xff\xcc"
"\x2c\xf1\x44\x3f\xdb\xc0\xbb\xbe\xf0\xed\x05\xbf\x0b\xff\xff\xcc"
"\x8d\x58\xd0\xbe\x2b\x92\x3a\x3f\x5d\xfb\x0c\x3f\x0b\xff\xff\xcc"
"\xb5\x24\x80\xbd\x84\x6a\x1a\xbf\x95\x8e\x4b\x3f\x0b\xff\xff\xe5"
"\x38\x19\xac\xbe\x80\xd6\x11\x3e\x3e\x55\x6e\xbf\x0b\xff\xff\xe5"
"\x9e\x9a\x03\x3f\xe4\x47\xf6\x3d\x18\x6a\x59\x3f\x0b\xff\xff\xe5"
"\x1c\xa3\x35\x3f\x75\x88\xce\x3e\x36\xeb\x13\xbf\x0b\xff\xff\xe5"
"\x38\x83\x5c\xbf\x49\xf3\xd0\x3e\xb4\xda\x9a\xbe\x0b\xff\xff\xe5"
"\x95\xe7\x9d\xbc\xf1\x8c\xa6\xbe\x06\x07\x72\x3f\x0b\xe5\xe5\xff"
"\x9b\xbb\xcf\x3c\xb2\x10\xc2\xbe\x84\xcf\x6c\xbf\x0b\xe5\xe5\xff"
"\x19\x0d\xbd\xbe\xb3\xe8\x6d\xbf\xcc\x52\x5a\xbb\x0b\xe5\xe5\xff"
"\x49\xc6\x00\xbf\x0e\x3b\x5d\xbf\x8a\x37\x4f\xbc\x0b\xe5\xe5\xff"
"\x0f\x32\xf6\xbc\x92\x16\x45\xbf\x8b\x32\x23\x3f\x0b\xe5\xe5\xff"
"\x56\xcf\x50\xbc\x8e\xdb\x14\xbe\xc8\x42\x7d\xbf\x0b\xe5\xe5\xff"
"\xdb\x61\xc5\xbe\xa9\x41\x1e\xbf\x32\x5c\x2f\x3f\x0b\xe5\xe5\xff"
"\x18\x7f\xc7\x3e\x48\x92\x58\x3f\xaf\x5e\xba\xbe\x0b\xe5\xe5\xff"
"\xa9\x73\x79\x3f\xe9\x8f\x24\x3e\x8b\xd8\x20\x3e\x0b\xe5\xe5\xff"
"\x65\x95\x20\xbf\xd6\x2c\x16\xbf\x97\x22\x03\x3f\x0b\xe5\xe5\xff"
"\x79\xc1\x9a\xbe\x16\x2e\x07\xbf\xcb\x29\x4b\x3f\x0b\xe5\xe5\xff"
"\xff\xc1\xf1\xbe\xd9\x59\x5c\xbf\x7e\xbd\x42\xbe\x0b\xe5\xe5\xff"
"\xdd\xed\xdd\x3e\x43\x8f\x33\xbc\x3f\xaf\x66\x3f\x0b\xe5\xe5\xff"
"\x96\x11\xad\x3e\x5d\xf6\x1c\x3f\xa2\xc9\x36\x3f\x0b\xe5\xe5\xff"
"\xe7\xa6\xb2\x3e\x96\x31\xd4\x3d\xb0\x70\x6e\xbf\x0b\xe5\xe5\xff"
"\x04\x35\xd3\xbe\x81\x3b\x53\xbf\xc1\xa3\xc5\xbe\x0b\xe5\xe5\xff"
"\xb2\x96\xa6\x3d\x91\xd1\x60\xbf\xe4\x53\xf1\xbe\x0b\xff\xff\xff"
"\xd7\x14\x5a\xbe\x6d\x1f\x6a\xbf\xbf\x13\xb0\xbe\x0b\xff\xff\xff"
"\x9b\x16\x71\x3f\x13\xd6\x9a\xbe\x27\xa8\x16\x3e\x0b\xff\xff\xff"
"\x0e\x06\xfb\xbe\xff\xf0\x5e\x3f\xbb\x5f\x0f\x3d\x0b\xff\xff\xff"
"\x12\x42\x73\xbf\x30\x0c\x9c\x3e\x23\x4e\x84\xbd\x0b\xff\xff\xff"
"\xd8\x21\x56\xbf\x8e\x24\xae\x3d\x2c\x99\x0a\x3f\x0b\xff\xff\xff"
"\x8f\x13\x70\x3e\x73\xa1\x27\x3e\xd2\x4f\x75\xbf\x0b\xff\xff\xff"
"\xfa\xc7\xd5\x3e\x66\x3a\x3a\xbf\x54\x64\x0b\xbf\x0b\xff\xff\xff"
"\xa0\x43\x8a\x3e\xb9\x86\x65\xbf\x53\xb8\xb3\x3e\x0b\xff\xff\xff"
"\xf3\xdb\x2b\xbf\xb3\xe9\x2f\x3f\xfb\x32\x8e\xbe\x0b\xff\xff\xff"
"\x19\xac\x23\x3f\x74\x7e\x5a\xbd\xac\x5e\x44\xbf\x0b\xff\xff\xff"
"\x3c\x05\x15\x3f\xeb\x4c\x4b\xbf\x23\xcc\x32\x3e\x0b\xff\xff\xff"
"\x31\x04\x36\x3f\x61\x08\x21\xbe\x35\x75\x2f\xbf\x0c\xff\xb2\x7f"
"\xe0\x43\x31\xbf\x13\xd5\x2b\x3f\x87\x73\x87\xbe\x0c\xff\xb2\x7f"
"\x52\xae\x28\x3e\x59\xa9\x05\xbf\xbe\x39\x56\x3f\x0c\xff\xb2\x7f"
"\xf7\x0c\x85\x3d\x45\xaf\x7c\xbf\xa2\x3a\x16\xbe\x0c\xff\xb2\x7f"
"\x11\xee\x2d\xbf\x2d\x8a\x39\x3f\x37\x89\xea\xbd\x0c\xff\xcc\xb2"
"\x94\x8b\xfb\x3e\xc7\xc9\x05\x3f\xa0\x60\x32\x3f\x0c\xff\xcc\xb2"
"\x41\x6b\x76\xbf\xe2\xc4\x7d\xbe\x94\xa4\xe0\xbd\x0c\xff\xcc\xb2"
"\x19\x19\xf4\x3e\x2f\xe7\x34\xbf\x25\xd8\x05\x3f\x0c\xff\xcc\xb2"
"\x2f\x49\x04\xbe\xc6\xfe\x28\xbf\x12\x6e\x3d\xbf\x0c\xff\xcc\xb2"
"\xfc\xa9\x2f\xbf\x23\xf0\x0c\x3f\x44\x6e\xf3\xbe\x0c\xff\xcc\xb2"
"\x78\x0d\x05\xbf\xf9\x88\x56\xbf\xdf\x16\x2a\xbe\x0c\xff\xcc\xb2"
"\xbf\x3b\xc2\xbd\x56\x46\x43\xbf\x73\xc0\x23\x3f\x0c\xff\xcc\xb2"
"\xe4\x28\x47\x3b\x60\x1f\x2e\x3f\xd5\xa9\x3b\x3f\x0c\xff\xcc\xb2"
"\x68\xaf\x48\x3d\xdc\x73\xf4\xbe\xc4\x96\x60\xbf\x0c\xff\xcc\xb2"
"\x38\x41\xc7\xbd\x28\xd5\x89\x3e\x12\x4a\x75\x3f\x0c\xff\xcc\xb2"
"\xb3\x04\x8d\x3d\xeb\x2a\x05\xbf\x34\xed\x59\x3f\x0c\xff\xcc\xb2"
"\x8a\x2e\x84\x3e\x03\x80\x75\x3f\x45\xc3\xef\xbd\x0c\xff\xcc\xb2"
"\xad\x58\xd4\xbe\x8e\x81\x3a\xbf\x8b\x91\x0b\xbf\x0c\xff\xcc\xb2"
"\xf1\x6e\x81\x3e\xdc\x72\x1f\x3d\x39\x7c\x77\x3f\x0c\xff\xcc\xb2"
"\x56\xba\x5f\x3e\xd0\x35\x72\x3f\x4f\xb4\x74\x3e\x0c\xff\xff\xcc"
"\xf2\xdc\x4a\xbf\x3a\x9d\x9e\x3d\x9e\xe2\x1a\xbf\x0c\xff\xff\xcc"
"\xd8\xc4\x40\x3f\x74\xdd\x03\x3f\x87\xa5\xd1\x3e\x0c\xff\xff\xcc"
"\x36\x96\x8d\xbe\x1e\xe7\xfe\x3c\x8d\xe3\x75\x3f\x0c\xff\xff\xcc"
"\xca\x6d\x62\x3d\x87\xa6\x10\xbf\x04\xbe\x52\x3f\x0c\xff\xff\xe5"
"\xa2\x0e\x2b\x3f\xc0\xfc\x59\xbe\x8d\x7f\x36\x3f\x0c\xff\xff\xe5"
"\x22\xb8\x03\xbf\x02\x5f\x46\x3f\x9d\xfd\xbb\xbe\x0c\xff\xff\xe5"
"\xff\xa0\x13\x3e\x80\x63\x3c\x3c\x3d\x4f\x7d\xbf\x0c\xff\xff\xe5"
"\xd0\x66\xa1\x3e\x7f\xc9\x28\x3f\xce\xbd\x2e\xbf\x0c\xe5\xe5\xff"
"\x75\x49\x35\x3f\x87\x06\x1b\xbf\x19\xe2\xb9\x3e\x0c\xe5\xe5\xff"
"\x8e\xa0\x76\xbe\xb4\x3d\x13\xbf\x5b\x23\x48\xbf\x0c\xe5\xe5\xff"
"\x7d\xe7\xd0\xbe\x0a\xbe\xf0\xbe\xb2\x57\x48\xbf\x0c\xe5\xe5\xff"
"\x70\xb9\x1e\xbf\x0e\x02\x19\xbf\xb5\x1e\x02\x3f\x0c\xe5\xe5\xff"
"\xe0\xdb\xc7\xbe\x1a\x52\x64\xbf\x70\xed\x69\x3e\x0c\xe5\xe5\xff"
"\xce\xcb\x06\xbe\xf3\xb3\x06\xbf\x5b\x12\x57\x3f\x0c\xe5\xe5\xff"
"\xca\x8e\xfc\xbe\xaf\xa2\x5c\xbf\x5b\x53\xf1\x3d\x0c\xe5\xe5\xff"
"\x96\x4c\xed\xbe\x1f\x59\x15\xbf\x87\xbe\x2a\x3f\x0c\xe5\xe5\xff"
"\x94\x78\xa5\x3d\xf4\x04\x13\x3f\x3e\x8d\x50\xbf\x0c\xe5\xe5\xff"
"\xa0\x20\x89\xbe\x33\xc5\x3d\xbf\x9b\x8d\x1d\xbf\x0c\xe5\xe5\xff"
"\x4e\x3d\x95\x3c\xbc\x16\x73\x3f\x07\x49\xa0\x3e\x0c\xe5\xe5\xff"
"\xd5\x20\x98\x3e\x5f\xb5\xc1\xbe\xa5\x6e\x60\x3f\x0c\xe5\xe5\xff"
"\xb6\x22\xd2\xbe\xbd\xb0\x0f\xbf\xdc\xfb\x37\x3f\x0c\xe5\xe5\xff"
"\xc5\x02\x2c\x3f\xfe\xc8\x04\x3d\x44\x6b\x3d\x3f\x0c\xe5\xe5\xff"
"\xfe\xd6\xab\x3e\x91\xa9\x70\x3f\xaf\xc4\x75\x3d\x0c\xe5\xe5\xff"
"\x17\x4b\xbd\x3e\xef\xcb\x19\x3f\x08\x74\x35\x3f\x0c\xe5\xe5\xff"
"\x1a\x49\x00\xbf\xf8\x48\x33\x3f\xa3\x23\x02\x3f\x0c\xff\xff\xff"
"\xe4\xd8\xe8\xbd\x4b\xaf\x7a\x3f\x7c\xe4\x2b\x3e\x0c\xff\xff\xff"
"\xbe\x0c\x02\xbf\x19\x11\x39\x3f\x78\xc7\xef\xbe\x0c\xff\xff\xff"
"\xf7\x95\x0a\xbc\x5c\x60\x03\x3f\xac\xb5\x5b\x3f\x0c\xff\xff\xff"
"\xbb\x0b\xa0\xbe\xea\x9b\x5a\xbf\x38\x01\xd5\x3e\x0c\xff\xff\xff"
"\x20\x73\x80\xbd\x88\x40\x50\xbe\xf4\x22\x7a\xbf\x0d\xb2\xcc\xff"
"\xf1\xfd\x95\xbe\xf5\xaa\x48\x3d\xac\x72\x74\x3f\x0d\xff\xb2\x7f"
"\x54\xa5\x66\xbe\x28\xdb\x98\x3e\x9d\x6c\x6d\xbf\x0d\xff\xb2\x7f"
"\x48\xba\xac\x3e\xe2\x88\xaf\xbe\x79\x71\x60\xbf\x0d\xff\xb2\x7f"
"\xbc\x41\xc6\x3d\x56\xda\x36\x3e\x28\xaa\x7a\x3f\x0d\xff\xcc\xb2"
"\x19\x06\x72\x3e\x24\x09\xb2\xbe\x23\x46\x68\x3f\x0d\xff\xcc\xb2"
"\xd0\x8f\xf1\xbc\x1c\xa1\x42\xbf\xf9\x20\x26\x3f\x0d\xff\xcc\xb2"
"\x12\x99\xdf\x3e\x7e\xcd\x5e\xbf\x1c\x1c\x69\x3e\x0d\xff\xcc\xb2"
"\xc3\x94\x8b\xbe\x81\x8f\x51\xbf\x5d\x6c\x01\xbf\x0d\xff\xcc\xb2"
"\x76\xcb\x0f\x3e\x74\x06\xd8\xbe\x26\x4c\x65\x3f\x0d\xff\xcc\xb2"
"\x55\x63\x6b\xbf\x6f\x2e\xa4\xbe\x75\xdf\x68\x3e\x0d\xff\xcc\xb2"
"\xa7\xa2\xa1\xbd\x3a\x4c\x6d\x3f\xd8\xcf\xbb\x3e\x0d\xff\xff\xcc"
"\xe1\x39\x44\xbe\x0e\xb3\xef\x3e\xad\xd4\x5c\x3f\x0d\xff\xff\xcc"
"\x67\xf2\x70\x3d\x7b\x4c\x59\x3f\xfd\x80\x06\x3f\x0d\xff\xff\xcc"
"\x19\x38\x2b\x3e\x46\x29\x7c\xbf\x40\x8f\x2e\x3d\x0d\xff\xff\xcc"
"\x78\x87\x01\xbe\x8b\x6d\x1d\x3f\x9e\x42\x47\x3f\x0d\xff\xff\xcc"
"\x76\x34\x61\xbf\x9e\xbd\x8b\x3e\xe3\x5d\xc7\x3e\x0d\xff\xff\xcc"
"\xeb\xb0\x29\xbd\x9f\xba\xe5\xbe\x2e\x8a\x64\xbf\x0d\xff\xff\xcc"
"\x96\x5a\x60\x3f\x10\xaf\xb4\xbd\xda\x69\xf2\x3e\x0d\xff\xff\xcc"
"\x2f\x5c\xbe\x3e\x88\x68\x3b\x3f\x4c\x22\x12\xbf\x0d\xff\xff\xcc"
"\x03\xb5\x5f\x3f\xd1\xe8\xf3\x3e\x9d\xe4\xc6\x3d\x0d\xff\xff\xcc"
"\xa6\xa2\x71\x3f\x2d\x69\x6a\xbc\x27\xef\xa8\x3e\x0d\xff\xff\xcc"
"\xfc\x11\x98\xbe\x02\x1e\xb2\x3e\xda\xa5\x63\xbf\x0d\xff\xff\xcc"
"\x2b\x03\x51\xbf\xb0\xa4\xcc\x3e\x9d\x5a\xd5\xbe\x0d\xff\xff\xcc"
"\x77\xcc\x51\xbf\x87\x50\x9d\xbe\x09\xa8\xf7\x3e\x0d\xff\xff\xcc"
"\x7a\xf8\x1e\xbf\x0f\xd3\xe7\xbe\xba\xcb\x23\xbf\x0d\xff\xff\xe5"
"\x8b\x26\x53\xbe\x6c\x71\x56\xbf\xd1\x79\x01\x3f\x0d\xff\xff\xe5"
"\xfe\x0e\xa8\x3e\x17\x02\x6c\xbf\x7f\xbe\x52\xbe\x0d\xff\xff\xe5"
"\xcd\xf1\xcf\x3e\xfd\xb5\x93\xbe\xbf\xf8\x5d\x3f\x0d\xff\xff\xe5"
"\x9d\xbf\xc1\x3d\xa2\x1f\x1c\x3f\xd4\x6e\x49\xbf\x0d\xff\xff\xe5"
"\xa9\xf4\xee\xbe\x28\x5b\x62\xbf\xc2\xfe\x9d\x3c\x0d\xe5\xe5\xff"
"\xf6\x7f\xbc\xbe\xef\x85\x11\xbf\xed\x59\x3c\x3f\x0d\xe5\xe5\xff"
"\x7c\x22\x0b\xbf\xd2\x5d\x50\xbf\x2d\x30\x52\x3e\x0d\xe5\xe5\xff"
"\x9d\xb0\x4a\xbe\x63\x2f\xbe\xbe\x48\x38\x68\x3f\x0d\xe5\xe5\xff"
"\xcb\xbf\x73\x3f\x33\xa6\x47\x3e\x59\x06\x71\xbe\x0d\xe5\xe5\xff"
"\xc0\xd9\xae\x3e\xed\x23\xdb\x3e\xb0\x36\x56\xbf\x0d\xe5\xe5\xff"
"\x88\xc3\xc4\x3e\xf3\x79\x9a\x3e\x3b\x5d\x5f\xbf\x0d\xe5\xe5\xff"
"\x37\x5d\xe3\x3e\xce\xc1\x3e\x3f\xb3\xc0\xfe\x3e\x0d\xe5\xe5\xff"
"\x61\xc0\x9b\x3e\x87\x33\x0d\x3f\xc9\xd4\x46\xbf\x0d\xe5\xe5\xff"
"\x02\x40\xad\xbe\x68\xa9\xab\xbe\x06\x17\x61\x3f\x0d\xe5\xe5\xff"
"\x37\x11\x83\xbe\xdb\xb6\xa7\xbe\x67\xd4\x68\x3f\x0d\xe5\xe5\xff"
"\x49\xc3\x01\xbf\xc1\x44\xa0\x3d\xc7\xc3\x5b\xbf\x0d\xe5\xe5\xff"
"\x2b\xd5\x6d\xbf\x62\x9a\x58\xbe\x34\x6c\x9b\x3e\x0d\xe5\xe5\xff"
"\xda\x8a\xf0\xbb\x71\x97\x31\xbf\xed\x5f\x38\x3f\x0d\xe5\xe5\xff"
"\x2c\xec\x40\xbd\xbd\x58\x87\x3e\x7f\x99\x76\xbf\x0d\xe5\xe5\xff"
"\x91\x22\x39\xbf\xcb\x50\x05\xbf\x80\x46\xe8\x3e\x0d\xff\xff\xff"
"\x72\xd5\x19\xbf\xee\x5f\x2c\xbe\x20\x09\x48\x3f\x0d\xff\xff\xff"
"\xd0\xf8\x6b\x3d\x93\x36\x47\xbe\x36\xad\x7a\xbf\x0d\xff\xff\xff"
"\x3b\xa5\x5d\x3f\xfc\x28\x8d\xbe\xb1\xcc\xd5\x3e\x0d\xff\xff\xff"
"\x50\xdc\x69\xbe\xe9\x6a\x62\xbf\xa9\x5b\xd0\x3e\x0d\xff\xff\xff"
"\x86\xb1\x30\xbf\xcd\x77\x2e\x3f\x67\x0c\x79\x3e\x0d\xff\xff\xff"
"\x7a\xb7\xda\xbe\x33\xa6\x66\xbf\xaa\x92\x9b\xbd\x0d\xff\xff\xff"
"\x63\x1a\xde\x3e\x64\xbc\x9e\x3e\x3e\x93\x58\x3f\x0d\xff\xff\xff"
"\xe7\xb7\x22\x3f\x21\x20\xf8\x3e\x70\xd7\x19\xbf\x0d\xff\xff\xff"
"\x5e\x6f\x66\x3f\x0e\xaa\xde\x3e\x37\x26\xcc\x3c\x0e\xff\xb2\x7f"
"\x9d\xe3\x12\x3f\x46\x79\x26\xbd\x60\x68\x51\xbf\x0e\xff\xb2\x7f"
"\x2e\x8b\x2b\x3f\x79\x83\x14\x3f\xc9\x16\xed\x3e\x0e\xff\xcc\xb2"
"\x67\xcf\x63\xbf\x2d\x5c\xb4\x3e\x32\x67\x94\xbe\x0e\xff\xcc\xb2"
"\xb4\x8a\xcf\xbe\xc6\x04\x66\x3e\xd1\xd9\x62\x3f\x0e\xff\xcc\xb2"
"\xa2\x53\xed\x3d\x92\x94\xe7\x3e\xdc\x61\x62\xbf\x0e\xff\xcc\xb2"
"\x3d\xa5\xd2\xbe\x84\x96\x27\xbf\x02\x5a\x22\xbf\x0e\xff\xcc\xb2"
"\xda\xb1\x0a\xbf\xda\xc1\x0c\x3f\x0e\xc0\x22\x3f\x0e\xff\xcc\xb2"
"\xfd\x39\x4a\x3e\x2b\x61\xd5\xbe\xb4\x26\x63\x3f\x0e\xff\xcc\xb2"
"\x56\x0d\x16\x3f\x3d\x5d\x75\x3e\xe3\x22\x46\x3f\x0e\xff\xcc\xb2"
"\x5d\xba\x63\xbf\xb6\x67\xa6\xbe\xba\x5a\xa4\x3e\x0e\xff\xff\xcc"
"\xef\x20\x3b\xbf\x23\x40\xd1\x3e\xef\xe6\x0b\x3f\x0e\xff\xff\xcc"
"\x21\x4f\x2d\xbf\x63\x52\x3c\xbf\x4b\x6a\xbd\xbc\x0e\xff\xff\xcc"
"\x4b\x15\x70\xbd\xf5\x79\xcd\xbe\x3e\x00\x6a\xbf\x0e\xff\xff\xe5"
"\xa1\xe9\xed\xbe\xbd\x59\x0b\xbf\x56\xcb\x32\xbf\x0e\xe5\xe5\xff"
"\x35\x99\xf9\x3e\x8c\x55\x53\x3f\xaf\x97\x91\xbe\x0e\xe5\xe5\xff"
"\x3f\x29\x79\x3f\x97\xa2\x65\x3e\x4a\x8c\x49\x3d\x0e\xe5\xe5\xff"
"\xef\xa0\x08\xbf\x25\x3f\x52\xbf\x76\x82\x4e\xbe\x0e\xe5\xe5\xff"
"\xe8\xa0\x10\xbf\x15\x67\x29\xbf\x4b\x5c\xfc\x3e\x0e\xe5\xe5\xff"
"\xf6\xaa\x70\xbf\xf5\xda\x6f\xbe\xa8\x95\x7d\x3e\x0e\xe5\xe5\xff"
"\x86\xb3\x27\xbf\x2b\x34\x3a\xbf\xed\x66\x51\x3e\x0e\xe5\xe5\xff"
"\x01\xc3\x04\xbf\xec\xf4\x4e\xbd\x9d\x80\x5a\x3f\x0e\xe5\xe5\xff"
"\xe0\xd7\xa5\xbe\x3a\x18\x6b\xbe\xa8\xf5\x6a\x3f\x0e\xe5\xe5\xff"
"\xbc\xa3\xb6\xbd\x0b\xc7\x23\xbf\xd9\x6d\x43\xbf\x0e\xe5\xe5\xff"
"\xbc\xa9\xea\xbe\x7e\x14\x19\xbf\x41\x55\x28\x3f\x0e\xe5\xe5\xff"
"\xe8\x6e\xc2\xbe\xa9\xa1\x66\xbf\x28\x33\x57\x3e\x0e\xe5\xe5\xff"
"\x23\x46\x35\x3b\x65\x43\x05\xbf\x34\x94\x5a\x3f\x0e\xe5\xe5\xff"
"\x9a\xb4\x79\xbd\x83\x60\x2a\xbe\x1c\xf3\x7b\x3f\x0e\xff\xff\xff"
"\x73\x79\x0c\xbf\x36\x03\x56\x3f\xb0\x64\xaf\xbb\x0e\xff\xff\xff"
"\x92\x11\xdf\x3e\x60\x74\x63\x3f\x4c\x91\x13\x3e\x0e\xff\xff\xff"
"\x8b\x3e\x6b\xbf\x95\x73\xc6\x3e\xe6\xb1\x95\xbd\x0e\xff\xff\xff"
"\x3c\xeb\x4b\xbe\xe9\x18\x95\xbd\x2b\x2e\x7a\x3f\x0e\xff\xff\xff"
"\x4a\xb9\xb0\x3e\x16\x0f\xdc\xbe\xce\x97\x55\x3f\x0e\xff\xff\xff"
"\xf1\x95\xd1\xbd\x89\x5c\x06\x3f\x7c\x53\x58\xbf\x0e\xff\xff\xff"
"\x99\x27\x40\x3f\x0d\xa9\x84\x3e\xd4\x9a\x1b\xbf\x0e\xff\xff\xff"
"\xb5\x32\xee\x3e\xb6\xf6\x50\x3f\x85\x56\xaf\xbe\x0e\xff\xff\xff"
"\xc1\x81\x06\xbf\xae\x41\x2d\xbf\x82\x02\x04\xbf\x0f\xb2\xcc\xff"
"\x63\xa1\x23\x3f\x10\x69\x3e\xbf\x21\x34\x48\x3e\x0f\xff\xb2\x7f"
"\xe9\xf1\x22\xbf\xf1\x84\x32\xbf\xe8\xb6\xa8\xbe\x0f\xff\xcc\xb2"
"\xf0\xa0\x0e\x3f\x95\x29\x49\x3f\x78\x81\x89\xbe\x0f\xff\xcc\xb2"
"\xfa\xaf\x04\x3e\x17\x5c\x56\x3f\xe8\xf6\x07\x3f\x0f\xff\xcc\xb2"
"\x33\xae\x17\x3f\xdc\xe9\x17\x3f\xe7\x77\x0b\xbf\x0f\xff\xcc\xb2"
"\x93\x4b\x0f\xbe\x55\x39\xf0\xbe\xb6\x37\x5f\x3f\x0f\xff\xcc\xb2"
"\x9c\xd8\x2d\x3f\xd4\xef\x38\xbf\x5a\x64\x05\xbe\x0f\xff\xcc\xb2"
"\xde\x50\x67\x3f\x50\x7a\x30\x3e\x38\xd2\xc8\x3e\x0f\xff\xcc\xb2"
"\xf7\xb5\x11\x3f\xdc\x74\x2c\x3f\xfd\x5b\xf1\x3e\x0f\xff\xff\xe5"
"\x4d\x72\x0a\x3f\xcb\xb0\x51\x3f\xd7\xe7\x43\xbe\x0f\xff\xff\xe5"
"\xf7\x75\xe4\x3e\x6d\x02\x11\x3f\x8a\x5e\x31\xbf\x0f\xe5\xe5\xff"
"\x43\x4a\x34\xbd\x5b\x86\xdc\xbd\x60\x43\x7e\xbf\x0f\xe5\xe5\xff"
"\x7c\xf8\x06\xbf\x79\x9a\x3d\xbf\xd8\x41\xd5\x3e\x0f\xe5\xe5\xff"
"\xb5\x0a\xf0\xbe\x25\x2f\x28\xbf\x81\x25\x17\x3f\x0f\xe5\xe5\xff"
"\x05\xca\x05\xbf\xf5\x15\x16\xbf\xd1\x76\x1e\x3f\x0f\xe5\xe5\xff"
"\xd1\xd1\xad\xbe\xf7\xc6\x8d\xbe\x2b\x20\x66\x3f\x0f\xe5\xe5\xff"
"\x1c\x93\x24\x3d\x48\x41\x0c\xbf\xe8\xe9\x55\x3f\x0f\xe5\xe5\xff"
"\x34\xd4\x72\xbf\x92\x7b\x9c\x3e\x9f\x5b\xa9\x3d\x0f\xff\xff\xff"
"\x73\xb6\x26\xbf\x30\xb6\x32\xbf\xfd\x61\x98\x3e\x0f\xff\xff\xff"
"\x82\xbe\xc3\xbe\x2b\x5e\x0c\x3f\x02\x68\x3e\x3f\x0f\xff\xff\xff"
"\xdd\x8e\xf3\xbe\x68\xa2\x5c\xbf\x91\x14\x34\xbe\x0f\xff\xff\xff"
"\x7e\x17\xea\xbe\x4b\xbb\x5d\x3f\x5d\xc2\x4e\x3e\x0f\xff\xff\xff"
"\xc3\x43\x03\xbf\xfa\xed\x58\x3f\x6e\x46\x0d\xbe\x0f\xff\xff\xff"
"\x06\xb5\xf8\xbe\xfc\x35\x43\xbf\xfe\xc5\xda\xbe\x10\x7f\xff\xff"
"\x56\x7b\xa9\xbd\x9a\x74\x09\xbe\x56\xcc\x7c\xbf\x10\xb2\xcc\xff"
"\xb6\x27\x40\x3f\x61\xcf\x01\x3f\x24\xe7\xd8\xbe\x10\xff\xb2\x7f"
"\x34\xb2\xdc\xbe\xc1\x86\x64\x3f\xd2\xc7\x06\xbe\x10\xff\xcc\xb2"
"\x5f\x18\xcc\xbe\x9d\x90\x60\xbf\x96\x04\x89\xbe\x10\xff\xcc\xb2"
"\xbe\xde\xc1\xbd\x6b\x6e\x63\xbf\x07\xfe\xe5\x3e\x10\xff\xcc\xb2"
"\x1f\xbb\x5f\xbf\x07\x3a\x23\xbe\x76\x13\xeb\xbe\x10\xff\xcc\xb2"
"\x71\xfb\x34\x3f\x5e\x1c\x9a\x3e\x56\xd7\x23\xbf\x10\xff\xcc\xb2"
"\x84\x60\x60\x3f\xdc\x80\xc3\xbe\xb9\x23\x96\xbe\x10\xff\xcc\xb2"
"\x2d\xf1\x21\xbf\xb4\x8c\x01\xbf\xa5\x17\x16\x3f\x10\xff\xcc\xb2"
"\xc4\x38\xfc\xbd\x6a\x64\x7c\x3f\x59\xf9\xe7\x3d\x10\xff\xcc\xb2"
"\xef\xff\xb3\x3e\x5d\xb5\x2f\x3f\x26\xfb\x22\xbf\x10\xff\xff\xe5"
"\x1d\x88\xd7\xbe\x98\xc7\x01\xbf\x08\x90\x40\xbf\x10\xff\xff\xe5"
"\xd8\x9d\x8f\x3c\x11\x84\x1a\xbf\xb4\x0f\x4c\x3f\x10\xff\xff\xe5"
"\xfc\xc1\x5e\x3d\xc2\xf6\x7d\x3f\x94\x8f\xe8\xbd\x10\xff\xff\xe5"
"\x9d\x59\x47\x3e\x64\xeb\xf9\xbe\x40\xcd\x59\x3f\x10\xe5\xe5\xff"
"\x41\xa4\xfa\xbe\xab\x4e\x51\x3f\xc7\x32\x9b\x3e\x10\xe5\xe5\xff"
"\xc4\xf2\xb5\x3e\xc2\x0d\x4f\xbf\xf6\xe7\xef\x3e\x10\xe5\xe5\xff"
"\x12\x8a\x86\xbe\x6a\x11\xc8\xbe\x95\xd7\x61\xbf\x10\xe5\xe5\xff"
"\xa9\xf1\xaa\x3e\xb9\xa1\xbd\xbe\x14\xe7\x5d\x3f\x10\xe5\xe5\xff"
"\x66\x9a\x60\x3f\x93\xef\xd9\x3e\x23\xd0\x62\xbe\x10\xe5\xe5\xff"
"\x71\x5c\xf1\xbd\x48\x3c\x87\xbe\x4a\x0f\x75\xbf\x10\xe5\xe5\xff"
"\xc2\x2d\xf3\xbe\x6d\x64\x5a\xbf\x6e\x2a\x5d\x3e\x10\xe5\xe5\xff"
"\xd2\x14\xf4\xbe\x44\x9f\x5e\x3f\xb2\x91\x03\x3e\x10\xff\xff\xff"
"\x33\x4a\xa9\xbd\xec\x60\x22\x3f\x41\xc7\x44\xbf\x10\xff\xff\xff"
"\x83\xb1\x98\xbe\x27\x97\x4c\x3e\x31\xf0\x6e\xbf\x10\xff\xff\xff"
"\x67\xb8\xf7\xbe\x9b\x2c\x54\xbf\x0f\xe8\x8f\xbe\x10\xff\xff\xff"
"\xc3\x65\x84\x3d\xab\x85\xa1\x3e\x43\x5d\x72\x3f\x10\xff\xff\xff"
"\xd2\x77\x0e\xbf\xfd\x31\x4b\xbf\xa3\x6b\x7b\x3e\x11\xff\xb2\x7f"
"\x87\x53\xc2\xbe\x86\x52\x0c\xbf\x73\xcd\x3e\xbf\x11\xe5\xe5\xff"
"\xc2\x2d\xf3\xbe\x6d\x64\x5a\xbf\x6e\x2a\x5d\x3e\x11\xe5\xe5\xff"
"\xdd\x20\x51\x3c\x1f\x8d\x04\xbf\xfb\xfc\x5a\x3f\x11\xe5\xe5\xff"
"\x84\x51\x0f\xbc\x66\x3d\xdc\x3b\x49\xfc\x7f\xbf\x11\xe5\xe5\xff"
"\x8e\x13\xbb\x3b\x4e\x00\xc5\x3e\xa6\x49\x6c\xbf\x11\xe5\xe5\xff"
"\x28\x33\x81\xbd\x91\x8b\xfa\xbd\x84\x90\x7d\xbf\x11\xe5\xe5\xff"
"\xad\xee\x1c\x3f\x54\xb8\x3b\xbf\xd1\x97\x96\x3e\x11\xe5\xe5\xff"
"\x3c\x09\xe1\xbe\x31\x7b\xf1\x3e\x9e\xb2\x43\xbf\x11\xff\xff\xff"
"\x3c\x09\xe1\xbe\x31\x7b\xf1\x3e\x9e\xb2\x43\xbf\x11\xff\xff\xff"
"\xbe\x05\xac\xbe\x16\x15\x70\xbf\xd9\xb8\xb2\xbd\x11\xff\xff\xff"
"\xcb\x2d\xfd\xbe\xf8\xe1\xd4\x3e\xa4\x66\x43\xbf\x12\xff\xcc\xb2"
"\x35\xa9\xb6\xbe\x5b\x83\x50\xbf\xf2\x3d\xea\x3e\x12\xff\xcc\xb2"
"\xc6\x54\x02\xbf\xa0\x20\x51\xbf\x9c\xca\x8a\x3e\x12\xe5\xe5\xff"
"\x6e\xee\x6a\xbf\x6b\x61\x5d\x3e\x16\xa5\xaa\xbe\x12\xe5\xe5\xff"
"\x15\x41\x57\x3f\x5e\x51\x05\xbf\xbd\x25\x17\x3e\x12\xff\xff\xff"
"\xe9\xd8\x11\x3f\x70\x97\x3a\x3f\x66\x68\xc2\x3e\x12\xff\xff\xff"
"\x75\x05\x65\xbe\x8d\x0d\xaf\xbe\x20\xa9\x69\x3f\x13\xff\xb2\x7f"
"\xdf\xfc\x50\x3e\x9e\xef\x33\x3e\xd7\x8a\x76\xbf\x13\xff\xcc\xb2"
"\x8e\xed\xd4\xbe\xcc\x14\x50\xbf\xf0\xd7\xd0\x3e\x13\xe5\xe5\xff"
"\x5c\xc0\x59\xbf\x00\xb7\xcf\xbd\x5f\x15\x04\x3f\x13\xe5\xe5\xff"
"\x82\x84\x1a\x3f\xdd\x4b\x55\x3e\x4d\x05\x45\x3f\x13\xff\xff\xff"
"\x7b\x98\x0c\xbe\x9a\x71\x03\x3d\x85\x71\x7d\xbf\x14\xff\xb2\x7f"
"\x6c\x8d\x0e\xbf\x60\xc7\xf4\x3c\x2f\x80\x54\xbf\x14\xff\xff\xe5"
"\x2d\xe6\xd1\x3e\xba\x31\x63\xbf\x9c\x96\x57\xbe\x14\xe5\xe5\xff"
"\x6d\x90\x2b\xbf\x19\xdd\xd9\x3e\xc1\xad\x1b\x3f\x15\xff\xcc\xb2"
"\x35\xa9\xb6\xbe\x5b\x83\x50\xbf\xf2\x3d\xea\x3e\x15\xff\xff\xcc"
"\x4d\x28\x48\x3d\x15\x3c\x24\x3f\xfd\xf9\x43\xbf\x15\xff\xff\xcc"
"\x8d\xa4\xd4\x3c\xc5\x58\x7c\xbe\x6e\x04\x78\xbf\x15\xe5\xe5\xff"
"\x98\xb5\x8e\x3e\x1a\x1f\x30\x3f\x9e\x8a\x2b\x3f\x15\xff\xff\xff"
"\xf1\xc4\x4a\xbe\x57\x05\x58\xbf\xe7\x5b\xff\xbe\x16\xff\xff\xe5"
"\x13\xa3\xb4\xbe\x23\x4b\xba\xbe\xee\xaf\x5c\xbf\x18\xff\xff\xff";

} // namespace context
} // namespace isight
